@@ -39,6 +39,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <locale.h>
+#include <errno.h>
 #include <ctype.h>		/* For tolower() */
 #if !defined (HAVE_STRSIGNAL) || !defined(NO_SYS_SIGLIST_DECL)
 #include <signal.h>
@@ -133,22 +134,19 @@ gchar*
 g_strnfill (gsize length,     
 	    gchar fill_char)
 {
-  register gchar *str, *s, *end;
+  gchar *str;
 
   str = g_new (gchar, length + 1);
-  s = str;
-  end = str + length;
-  while (s < end)
-    *(s++) = fill_char;
-  *s = 0;
+  memset (str, (guchar)fill_char, length);
+  str[length] = '\0';
 
   return str;
 }
 
 /**
  * g_stpcpy:
- * @dest: destination buffer
- * @src: source string
+ * @dest: destination buffer.
+ * @src: source string.
  * 
  * Copies a nul-terminated string into the dest buffer, include the
  * trailing nul, and return a pointer to the trailing nul byte.
@@ -186,7 +184,7 @@ g_strdup_vprintf (const gchar *format,
   gchar *buffer;
 #ifdef HAVE_VASPRINTF
   vasprintf (&buffer, format, args1);
-  if (g_mem_is_system_malloc ()) 
+  if (!g_mem_is_system_malloc ()) 
     {
       gchar *buffer1 = g_strdup (buffer);
       free (buffer);
@@ -256,9 +254,29 @@ g_strconcat (const gchar *string1, ...)
   return concat;
 }
 
+/**
+ * g_strtod:
+ * @nptr:    the string to convert to a numeric value.
+ * @endptr:  if non-NULL, it returns the character after
+ *           the last character used in the conversion.
+ * 
+ * Converts a string to a gdouble value.
+ * It calls the standard strtod() function to handle the conversion, but
+ * if the string is not completely converted it attempts the conversion
+ * again with @g_ascii_strtod, and returns the best match.
+ *
+ * This function should seldom be used. The normal situation when reading
+ * numbers not for human consumption is to use @g_ascii_strtod(). Only when
+ * you know that you must expect both locale formated and C formated numbers
+ * should you use this. Make sure that you don't pass strings such as comma
+ * separated lists of values, since the commas may be interpreted as a decimal
+ * point in some locales, causing unexpected results.
+ * 
+ * Return value: the gdouble value.
+ **/
 gdouble
 g_strtod (const gchar *nptr,
-	  gchar **endptr)
+	  gchar      **endptr)
 {
   gchar *fail_pos_1;
   gchar *fail_pos_2;
@@ -273,15 +291,7 @@ g_strtod (const gchar *nptr,
   val_1 = strtod (nptr, &fail_pos_1);
 
   if (fail_pos_1 && fail_pos_1[0] != 0)
-    {
-      gchar *old_locale;
-
-      old_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
-      setlocale (LC_NUMERIC, "C");
-      val_2 = strtod (nptr, &fail_pos_2);
-      setlocale (LC_NUMERIC, old_locale);
-      g_free (old_locale);
-    }
+    val_2 = g_ascii_strtod (nptr, &fail_pos_2);
 
   if (!fail_pos_1 || fail_pos_1[0] == 0 || fail_pos_1 >= fail_pos_2)
     {
@@ -297,6 +307,278 @@ g_strtod (const gchar *nptr,
     }
 }
 
+/**
+ * g_ascii_strtod:
+ * @nptr:    the string to convert to a numeric value.
+ * @endptr:  if non-NULL, it returns the character after
+ *           the last character used in the conversion.
+ * 
+ * Converts a string to a gdouble value.
+ * This function behaves like the standard strtod() function
+ * does in the C locale. It does this without actually
+ * changing the current locale, since that would not be
+ * thread-safe.
+ *
+ * This function is typically used when reading configuration
+ * files or other non-user input that should be locale dependent.
+ * To handle input from the user you should normally use the
+ * locale-sensitive system strtod function.
+ *
+ * To convert from a string to double in a locale-insensitive
+ * way, use @g_ascii_dtostr.
+ *
+ * If the correct value would cause overflow, plus or minus HUGE_VAL
+ * is returned (according to the sign of the value), and ERANGE is
+ * stored in errno. If the correct value would cause underflow,
+ * zero is returned and ERANGE is stored in errno.
+ * 
+ * This function resets errno before calling strtod() so that
+ * you can reliably detect overflow and underflow.
+ *
+ * Return value: the gdouble value.
+ **/
+gdouble
+g_ascii_strtod (const gchar *nptr,
+		gchar      **endptr)
+{
+  gchar *fail_pos;
+  gdouble val;
+  struct lconv *locale_data;
+  const char *decimal_point;
+  int decimal_point_len;
+  const char *p, *decimal_point_pos;
+  const char *end = NULL; /* Silence gcc */
+
+  g_return_val_if_fail (nptr != NULL, 0);
+
+  fail_pos = NULL;
+
+  locale_data = localeconv ();
+  decimal_point = locale_data->decimal_point;
+  decimal_point_len = strlen (decimal_point);
+
+  g_assert (decimal_point_len != 0);
+  
+  decimal_point_pos = NULL;
+  if (decimal_point[0] != '.' ||
+      decimal_point[1] != 0)
+    {
+      p = nptr;
+      /* Skip leading space */
+      while (isspace ((guchar)*p))
+	p++;
+      
+      /* Skip leading optional sign */
+      if (*p == '+' || *p == '-')
+	p++;
+      
+      if (p[0] == '0' &&
+	  (p[1] == 'x' || p[1] == 'X'))
+	{
+	  p += 2;
+	  /* HEX - find the (optional) decimal point */
+	  
+	  while (isxdigit ((guchar)*p))
+	    p++;
+	  
+	  if (*p == '.')
+	    {
+	      decimal_point_pos = p++;
+	      
+	      while (isxdigit ((guchar)*p))
+		p++;
+	      
+	      if (*p == 'p' || *p == 'P')
+		p++;
+	      if (*p == '+' || *p == '-')
+		p++;
+	      while (isdigit ((guchar)*p))
+		p++;
+	      end = p;
+	    }
+	}
+      else
+	{
+	  while (isdigit ((guchar)*p))
+	    p++;
+	  
+	  if (*p == '.')
+	    {
+	      decimal_point_pos = p++;
+	      
+	      while (isdigit ((guchar)*p))
+		p++;
+	      
+	      if (*p == 'e' || *p == 'E')
+		p++;
+	      if (*p == '+' || *p == '-')
+		p++;
+	      while (isdigit ((guchar)*p))
+		p++;
+	      end = p;
+	    }
+	}
+      /* For the other cases, we need not convert the decimal point */
+    }
+
+  /* Set errno to zero, so that we can distinguish zero results
+     and underflows */
+  errno = 0;
+  
+  if (decimal_point_pos)
+    {
+      char *copy, *c;
+
+      /* We need to convert the '.' to the locale specific decimal point */
+      copy = g_malloc (end - nptr + 1 + decimal_point_len);
+      
+      c = copy;
+      memcpy (c, nptr, decimal_point_pos - nptr);
+      c += decimal_point_pos - nptr;
+      memcpy (c, decimal_point, decimal_point_len);
+      c += decimal_point_len;
+      memcpy (c, decimal_point_pos + 1, end - (decimal_point_pos + 1));
+      c += end - (decimal_point_pos + 1);
+      *c = 0;
+
+      val = strtod (copy, &fail_pos);
+
+      if (fail_pos)
+	{
+	  if (fail_pos > decimal_point_pos)
+	    fail_pos = (char *)nptr + (fail_pos - copy) - (decimal_point_len - 1);
+	  else
+	    fail_pos = (char *)nptr + (fail_pos - copy);
+	}
+      
+      g_free (copy);
+	  
+    }
+  else
+    val = strtod (nptr, &fail_pos);
+
+  if (endptr)
+    *endptr = fail_pos;
+  
+  return val;
+}
+
+/**
+ * g_ascii_dtostr:
+ * @buffer: A buffer to place the resulting string in
+ * @buf_len: The length of the buffer.
+ * @d: The double to convert
+ *
+ * Converts a double to a string, using the '.' as
+ * decimal_point. 
+ * 
+ * This functions generates enough precision that converting
+ * the string back using @g_strtod gives the same machine-number
+ * (on machines with IEEE compatible 64bit doubles). It is
+ * guaranteed that the size of the resulting string will never
+ * be larger than @G_ASCII_DTOSTR_BUF_SIZE bytes.
+ *
+ * Return value: The pointer to the buffer with the converted string.
+ **/
+gchar *
+g_ascii_dtostr (gchar       *buffer,
+		gint         buf_len,
+		gdouble      d)
+{
+  return g_ascii_formatd (buffer, buf_len, "%.17g", d);
+}
+
+/**
+ * g_ascii_formatd:
+ * @buffer: A buffer to place the resulting string in
+ * @buf_len: The length of the buffer.
+ * @format: The printf-style format to use for the
+ *          code to use for converting. 
+ * @d: The double to convert
+ *
+ * Converts a double to a string, using the '.' as
+ * decimal_point. To format the number you pass in
+ * a printf-style formating string. Allowed conversion
+ * specifiers are eEfFgG. 
+ * 
+ * If you just want to want to serialize the value into a
+ * string, use @g_ascii_dtostr.
+ *
+ * Return value: The pointer to the buffer with the converted string.
+ **/
+gchar *
+g_ascii_formatd (gchar       *buffer,
+		 gint         buf_len,
+		 const gchar *format,
+		 gdouble      d)
+{
+  struct lconv *locale_data;
+  const char *decimal_point;
+  int decimal_point_len;
+  gchar *p;
+  int rest_len;
+  gchar format_char;
+
+  g_return_val_if_fail (buffer != NULL, NULL);
+  g_return_val_if_fail (format[0] == '%', NULL);
+  g_return_val_if_fail (strpbrk (format + 1, "'l%") == NULL, NULL);
+ 
+  format_char = format[strlen (format) - 1];
+  
+  g_return_val_if_fail (format_char == 'e' || format_char == 'E' ||
+			format_char == 'f' || format_char == 'F' ||
+			format_char == 'g' || format_char == 'G',
+			NULL);
+
+  if (format[0] != '%')
+    return NULL;
+
+  if (strpbrk (format + 1, "'l%"))
+    return NULL;
+
+  if (!(format_char == 'e' || format_char == 'E' ||
+	format_char == 'f' || format_char == 'F' ||
+	format_char == 'g' || format_char == 'G'))
+    return NULL;
+
+      
+  g_snprintf (buffer, buf_len, format, d);
+
+  locale_data = localeconv ();
+  decimal_point = locale_data->decimal_point;
+  decimal_point_len = strlen (decimal_point);
+
+  g_assert (decimal_point_len != 0);
+
+  if (decimal_point[0] != '.' ||
+      decimal_point[1] != 0)
+    {
+      p = buffer;
+
+      if (*p == '+' || *p == '-')
+	p++;
+
+      while (isdigit ((guchar)*p))
+	p++;
+
+      if (strncmp (p, decimal_point, decimal_point_len) == 0)
+	{
+	  *p = '.';
+	  p++;
+	  if (decimal_point_len > 1) {
+	    rest_len = strlen (p + (decimal_point_len-1));
+	    memmove (p, p + (decimal_point_len-1),
+		     rest_len);
+	    p[rest_len] = 0;
+	    
+	  }
+	}
+    }
+  
+  return buffer;
+}
+
+
 G_CONST_RETURN gchar*
 g_strerror (gint errnum)
 {
@@ -999,20 +1281,20 @@ g_strlcat (gchar       *dest,
 
 /**
  * g_ascii_strdown:
- * @str: a string
+ * @str: a string.
  * @len: length of @str in bytes, or -1 if @str is nul-terminated.
  * 
  * Converts all upper case ASCII letters to lower case ASCII letters.
  * 
- * Return value: a newly allocated string, with all the upper case
+ * Return value: a newly-allocated string, with all the upper case
  *               characters in @str converted to lower case, with
- *               semantics that exactly match g_ascii_tolower. (Note
- *               that this is unlike the old g_strdown, which modified
+ *               semantics that exactly match g_ascii_tolower(). (Note
+ *               that this is unlike the old g_strdown(), which modified
  *               the string in place.)
  **/
 gchar*
 g_ascii_strdown (const gchar *str,
-		 gint         len)
+		 gssize       len)
 {
   gchar *result, *s;
   
@@ -1030,20 +1312,20 @@ g_ascii_strdown (const gchar *str,
 
 /**
  * g_ascii_strup:
- * @str: a string
+ * @str: a string.
  * @len: length of @str in bytes, or -1 if @str is nul-terminated.
  * 
  * Converts all lower case ASCII letters to upper case ASCII letters.
  * 
  * Return value: a newly allocated string, with all the lower case
  *               characters in @str converted to upper case, with
- *               semantics that exactly match g_ascii_toupper. (Note
- *               that this is unlike the old g_strup, which modified
+ *               semantics that exactly match g_ascii_toupper(). (Note
+ *               that this is unlike the old g_strup(), which modified
  *               the string in place.)
  **/
 gchar*
 g_ascii_strup (const gchar *str,
-	       gint         len)
+	       gssize       len)
 {
   gchar *result, *s;
 
@@ -1126,16 +1408,16 @@ g_strreverse (gchar *string)
 
 /**
  * g_ascii_tolower:
- * @c: any character
+ * @c: any character.
  * 
  * Convert a character to ASCII lower case.
  *
- * Unlike the standard C library tolower function, this only
+ * Unlike the standard C library tolower() function, this only
  * recognizes standard ASCII letters and ignores the locale, returning
  * all non-ASCII characters unchanged, even if they are lower case
  * letters in a particular character set. Also unlike the standard
  * library function, this takes and returns a char, not an int, so
- * don't call it on EOF but no need to worry about casting to guchar
+ * don't call it on %EOF but no need to worry about casting to #guchar
  * before passing a possibly non-ASCII character in.
  * 
  * Return value: the result of converting @c to lower case.
@@ -1150,16 +1432,16 @@ g_ascii_tolower (gchar c)
 
 /**
  * g_ascii_toupper:
- * @c: any character
+ * @c: any character.
  * 
  * Convert a character to ASCII upper case.
  *
- * Unlike the standard C library toupper function, this only
+ * Unlike the standard C library toupper() function, this only
  * recognizes standard ASCII letters and ignores the locale, returning
  * all non-ASCII characters unchanged, even if they are upper case
  * letters in a particular character set. Also unlike the standard
  * library function, this takes and returns a char, not an int, so
- * don't call it on EOF but no need to worry about casting to guchar
+ * don't call it on %EOF but no need to worry about casting to #guchar
  * before passing a possibly non-ASCII character in.
  * 
  * Return value: the result of converting @c to upper case.
@@ -1174,15 +1456,15 @@ g_ascii_toupper (gchar c)
 
 /**
  * g_ascii_digit_value:
- * @c: an ASCII character
+ * @c: an ASCII character.
  *
  * Determines the numeric value of a character as a decimal
- * digit. Differs from g_unichar_digit_value because it takes
+ * digit. Differs from g_unichar_digit_value() because it takes
  * a char, so there's no worry about sign extension if characters
  * are signed.
  *
  * Return value: If @c is a decimal digit (according to
- * `g_ascii_isdigit'), its numeric value. Otherwise, -1.
+ * g_ascii_isdigit()), its numeric value. Otherwise, -1.
  **/
 int
 g_ascii_digit_value (gchar c)
@@ -1194,15 +1476,15 @@ g_ascii_digit_value (gchar c)
 
 /**
  * g_ascii_xdigit_value:
- * @c: an ASCII character
+ * @c: an ASCII character.
  *
  * Determines the numeric value of a character as a hexidecimal
- * digit. Differs from g_unichar_xdigit_value because it takes
+ * digit. Differs from g_unichar_xdigit_value() because it takes
  * a char, so there's no worry about sign extension if characters
  * are signed.
  *
  * Return value: If @c is a hex digit (according to
- * `g_ascii_isxdigit'), its numeric value. Otherwise, -1.
+ * g_ascii_isxdigit()), its numeric value. Otherwise, -1.
  **/
 int
 g_ascii_xdigit_value (gchar c)
@@ -1216,12 +1498,12 @@ g_ascii_xdigit_value (gchar c)
 
 /**
  * g_ascii_strcasecmp:
- * @s1: string to compare with @s2
- * @s2: string to compare with @s1
+ * @s1: string to compare with @s2.
+ * @s2: string to compare with @s1.
  * 
  * Compare two strings, ignoring the case of ASCII characters.
  *
- * Unlike the BSD strcasecmp function, this only recognizes standard
+ * Unlike the BSD strcasecmp() function, this only recognizes standard
  * ASCII letters and ignores the locale, treating all non-ASCII
  * characters as if they are not letters.
  * 
@@ -1252,14 +1534,14 @@ g_ascii_strcasecmp (const gchar *s1,
 
 /**
  * g_ascii_strncasecmp:
- * @s1: string to compare with @s2
- * @s2: string to compare with @s1
- * @n:  number of characters to compare
+ * @s1: string to compare with @s2.
+ * @s2: string to compare with @s1.
+ * @n:  number of characters to compare.
  * 
  * Compare @s1 and @s2, ignoring the case of ASCII characters and any
  * characters after the first @n in each string.
  *
- * Unlike the BSD strcasecmp function, this only recognizes standard
+ * Unlike the BSD strcasecmp() function, this only recognizes standard
  * ASCII letters and ignores the locale, treating all non-ASCII
  * characters as if they are not letters.
  * 
@@ -1271,7 +1553,7 @@ g_ascii_strcasecmp (const gchar *s1,
 gint
 g_ascii_strncasecmp (const gchar *s1,
 		     const gchar *s2,
-		     guint n)
+		     gsize n)
 {
   gint c1, c2;
 
@@ -1572,7 +1854,7 @@ g_strchomp (gchar *string)
  * @string: a string to split.
  * @delimiter: a string which specifies the places at which to split the string.
  *     The delimiter is not included in any of the resulting strings, unless
- *     max_tokens is reached.
+ *     @max_tokens is reached.
  * @max_tokens: the maximum number of pieces to split @string into. If this is
  *              less than 1, the string is split completely.
  * 
@@ -1587,8 +1869,8 @@ g_strchomp (gchar *string)
  * to represent empty elements, you'll need to check for the empty string
  * before calling g_strsplit().
  * 
- * Return value: a newly-allocated %NULL-terminated array of strings. Use g_strfreev()
- *    to free it.
+ * Return value: a newly-allocated %NULL-terminated array of strings. Use 
+ *    g_strfreev() to free it.
  **/
 gchar**
 g_strsplit (const gchar *string,
@@ -1664,14 +1946,14 @@ g_strfreev (gchar **str_array)
 
 /**
  * g_strdupv:
- * @str_array: %NULL-terminated array of strings
+ * @str_array: %NULL-terminated array of strings.
  * 
  * Copies %NULL-terminated array of strings. The copy is a deep copy;
  * the new array should be freed by first freeing each string, then
  * the array itself. g_strfreev() does this for you. If called
  * on a %NULL value, g_strdupv() simply returns %NULL.
  * 
- * Return value: a new %NULL-terminated array of strings
+ * Return value: a new %NULL-terminated array of strings.
  **/
 gchar**
 g_strdupv (gchar **str_array)
@@ -1800,16 +2082,16 @@ g_strjoin (const gchar  *separator,
 
 /**
  * g_strstr_len:
- * @haystack: a string
- * @haystack_len: The maximum length of haystack
- * @needle: The string to search for.
+ * @haystack: a string.
+ * @haystack_len: the maximum length of @haystack.
+ * @needle: the string to search for.
  *
- * Searches the string haystack for the first occurrence
- * of the string needle, limiting the length of the search
- * to haystack_len. 
+ * Searches the string @haystack for the first occurrence
+ * of the string @needle, limiting the length of the search
+ * to @haystack_len. 
  *
- * Return value: A pointer to the found occurrence, or
- * NULL if not found.
+ * Return value: a pointer to the found occurrence, or
+ *    %NULL if not found.
  **/
 gchar *
 g_strstr_len (const gchar *haystack,
@@ -1853,15 +2135,15 @@ g_strstr_len (const gchar *haystack,
 }
 
 /**
- * g_strrstr_len:
- * @haystack: a nul-terminated string
- * @needle: The nul-terminated string to search for.
+ * g_strrstr:
+ * @haystack: a nul-terminated string.
+ * @needle: the nul-terminated string to search for.
  *
- * Searches the string haystack for the last occurrence
- * of the string needle.
+ * Searches the string @haystack for the last occurrence
+ * of the string @needle.
  *
- * Return value: A pointer to the found occurrence, or
- * NULL if not found.
+ * Return value: a pointer to the found occurrence, or
+ *    %NULL if not found.
  **/
 gchar *
 g_strrstr (const gchar *haystack,
@@ -1903,16 +2185,16 @@ g_strrstr (const gchar *haystack,
 
 /**
  * g_strrstr_len:
- * @haystack: a nul-terminated string
- * @haystack_len: The maximum length of haystack
- * @needle: The nul-terminated string to search for.
+ * @haystack: a nul-terminated string.
+ * @haystack_len: the maximum length of @haystack.
+ * @needle: the nul-terminated string to search for.
  *
- * Searches the string haystack for the last occurrence
- * of the string needle, limiting the length of the search
- * to haystack_len. 
+ * Searches the string @haystack for the last occurrence
+ * of the string @needle, limiting the length of the search
+ * to @haystack_len. 
  *
- * Return value: A pointer to the found occurrence, or
- * NULL if not found.
+ * Return value: a pointer to the found occurrence, or
+ *    %NULL if not found.
  **/
 gchar *
 g_strrstr_len (const gchar *haystack,
