@@ -93,148 +93,9 @@ g_win32_ftruncate (gint  fd,
   return 0;
 }
 
-DIR*
-g_win32_opendir (const char *dirname)
-{
-  DIR *result;
-  gchar *mask;
-  guint k;
-
-  g_return_val_if_fail (dirname != NULL, NULL);
-
-  result = g_new0 (DIR, 1);
-  result->find_file_data = g_new0 (WIN32_FIND_DATA, 1);
-  result->dir_name = g_strdup (dirname);
-  
-  k = strlen (result->dir_name);
-  if (k && result->dir_name[k - 1] == '\\')
-    {
-      result->dir_name[k - 1] = '\0';
-    }
-  mask = g_strdup_printf ("%s\\*", result->dir_name);
-
-  result->find_file_handle = (guint) FindFirstFile (mask,
-					     (LPWIN32_FIND_DATA) result->find_file_data);
-  g_free (mask);
-
-  if (result->find_file_handle == (guint) INVALID_HANDLE_VALUE)
-    {
-      int error = GetLastError ();
-
-      g_free (result->dir_name);
-      g_free (result->find_file_data);
-      g_free (result);
-      switch (error)
-	{
-	default:
-	  errno = EIO;
-	  return NULL;
-	}
-    }
-  result->just_opened = TRUE;
-
-  return result;
-}
-
-struct dirent*
-g_win32_readdir (DIR *dir)
-{
-  gchar *basename;
-
-  g_return_val_if_fail (dir != NULL, NULL);
-
-  if (dir->just_opened)
-    dir->just_opened = FALSE;
-  else
-    {
-      if (!FindNextFile ((HANDLE) dir->find_file_handle,
-			 (LPWIN32_FIND_DATA) dir->find_file_data))
-	{
-	  int error = GetLastError ();
-
-	  switch (error)
-	    {
-	    case ERROR_NO_MORE_FILES:
-	      return NULL;
-	    default:
-	      errno = EIO;
-	      return NULL;
-	    }
-	}
-    }
-  
-  basename = g_path_get_basename (((LPWIN32_FIND_DATA) dir->find_file_data)->cFileName);
-
-  strcpy (dir->readdir_result.d_name, basename);
-
-  g_free (basename);
-      
-  return &dir->readdir_result;
-}
-
-void
-g_win32_rewinddir (DIR *dir)
-{
-  gchar *mask;
-
-  g_return_if_fail (dir != NULL);
-
-  if (!FindClose ((HANDLE) dir->find_file_handle))
-    g_warning ("gwin_rewinddir(): FindClose() failed\n");
-
-  mask = g_strdup_printf ("%s\\*", dir->dir_name);
-  dir->find_file_handle = (guint) FindFirstFile (mask,
-					  (LPWIN32_FIND_DATA) dir->find_file_data);
-  g_free (mask);
-
-  if (dir->find_file_handle == (guint) INVALID_HANDLE_VALUE)
-    {
-      int error = GetLastError ();
-
-      switch (error)
-	{
-	default:
-	  errno = EIO;
-	  return;
-	}
-    }
-  dir->just_opened = TRUE;
-}  
-
-gint
-g_win32_closedir (DIR *dir)
-{
-  g_return_val_if_fail (dir != NULL, -1);
-
-  if (!FindClose ((HANDLE) dir->find_file_handle))
-    {
-      int error = GetLastError ();
-
-      switch (error)
-	{
-	default:
-	  errno = EIO; return -1;
-	}
-    }
-
-  g_free (dir->dir_name);
-  g_free (dir->find_file_data);
-  g_free (dir);
-
-  return 0;
-}
 #endif
 
-/* Mingw headers don't have latest language and sublanguage codes */
-#ifndef LANG_AFRIKAANS
-#define LANG_AFRIKAANS 0x36
-#endif
-#ifndef LANG_ALBANIAN
-#define LANG_ALBANIAN 0x1c
-#endif
-#ifndef LANG_ARABIC
-#define LANG_ARABIC 0x01
-#endif
+/* MSVC 5.0 headers don't have latest language and sublanguage codes */
 #ifndef LANG_ARMENIAN
 #define LANG_ARMENIAN 0x2b
 #endif
@@ -244,42 +105,18 @@ g_win32_closedir (DIR *dir)
 #ifndef LANG_AZERI
 #define LANG_AZERI 0x2c
 #endif
-#ifndef LANG_BASQUE
-#define LANG_BASQUE 0x2d
-#endif
-#ifndef LANG_BELARUSIAN
-#define LANG_BELARUSIAN 0x23
-#endif
 #ifndef LANG_BENGALI
 #define LANG_BENGALI 0x45
 #endif
-#ifndef LANG_CATALAN
-#define LANG_CATALAN 0x03
-#endif
-#ifndef LANG_ESTONIAN
-#define LANG_ESTONIAN 0x25
-#endif
-#ifndef LANG_FAEROESE
-#define LANG_FAEROESE 0x38
-#endif
-#ifndef LANG_FARSI
-#define LANG_FARSI 0x29
-#endif
 #ifndef LANG_GEORGIAN
 #define LANG_GEORGIAN 0x37
 #endif
 #ifndef LANG_GUJARATI
 #define LANG_GUJARATI 0x47
 #endif
-#ifndef LANG_HEBREW
-#define LANG_HEBREW 0x0d
-#endif
 #ifndef LANG_HINDI
 #define LANG_HINDI 0x39
 #endif
-#ifndef LANG_INDONESIAN
-#define LANG_INDONESIAN 0x21
-#endif
 #ifndef LANG_KANNADA
 #define LANG_KANNADA 0x4b
 #endif
@@ -292,12 +129,6 @@ g_win32_closedir (DIR *dir)
 #ifndef LANG_KONKANI
 #define LANG_KONKANI 0x57
 #endif
-#ifndef LANG_LATVIAN
-#define LANG_LATVIAN 0x26
-#endif
-#ifndef LANG_LITHUANIAN
-#define LANG_LITHUANIAN 0x27
-#endif
 #ifndef LANG_MACEDONIAN
 #define LANG_MACEDONIAN 0x2f
 #endif
@@ -325,15 +156,9 @@ g_win32_closedir (DIR *dir)
 #ifndef LANG_SANSKRIT
 #define LANG_SANSKRIT 0x4f
 #endif
-#ifndef LANG_SERBIAN
-#define LANG_SERBIAN 0x1a
-#endif
 #ifndef LANG_SINDHI
 #define LANG_SINDHI 0x59
 #endif
-#ifndef LANG_SLOVAK
-#define LANG_SLOVAK 0x1b
-#endif
 #ifndef LANG_SWAHILI
 #define LANG_SWAHILI 0x41
 #endif
@@ -346,192 +171,40 @@ g_win32_closedir (DIR *dir)
 #ifndef LANG_TELUGU
 #define LANG_TELUGU 0x4a
 #endif
-#ifndef LANG_THAI
-#define LANG_THAI 0x1e
-#endif
-#ifndef LANG_UKRAINIAN
-#define LANG_UKRAINIAN 0x22
-#endif
 #ifndef LANG_URDU
 #define LANG_URDU 0x20
 #endif
 #ifndef LANG_UZBEK
 #define LANG_UZBEK 0x43
 #endif
-#ifndef LANG_VIETNAMESE
-#define LANG_VIETNAMESE 0x2a
-#endif
-#ifndef SUBLANG_ARABIC_SAUDI_ARABIA
-#define SUBLANG_ARABIC_SAUDI_ARABIA 0x01
-#endif
-#ifndef SUBLANG_ARABIC_IRAQ
-#define SUBLANG_ARABIC_IRAQ 0x02
-#endif
-#ifndef SUBLANG_ARABIC_EGYPT
-#define SUBLANG_ARABIC_EGYPT 0x03
-#endif
-#ifndef SUBLANG_ARABIC_LIBYA
-#define SUBLANG_ARABIC_LIBYA 0x04
-#endif
-#ifndef SUBLANG_ARABIC_ALGERIA
-#define SUBLANG_ARABIC_ALGERIA 0x05
-#endif
-#ifndef SUBLANG_ARABIC_MOROCCO
-#define SUBLANG_ARABIC_MOROCCO 0x06
-#endif
-#ifndef SUBLANG_ARABIC_TUNISIA
-#define SUBLANG_ARABIC_TUNISIA 0x07
-#endif
-#ifndef SUBLANG_ARABIC_OMAN
-#define SUBLANG_ARABIC_OMAN 0x08
-#endif
-#ifndef SUBLANG_ARABIC_YEMEN
-#define SUBLANG_ARABIC_YEMEN 0x09
-#endif
-#ifndef SUBLANG_ARABIC_SYRIA
-#define SUBLANG_ARABIC_SYRIA 0x0a
-#endif
-#ifndef SUBLANG_ARABIC_JORDAN
-#define SUBLANG_ARABIC_JORDAN 0x0b
-#endif
-#ifndef SUBLANG_ARABIC_LEBANON
-#define SUBLANG_ARABIC_LEBANON 0x0c
-#endif
-#ifndef SUBLANG_ARABIC_KUWAIT
-#define SUBLANG_ARABIC_KUWAIT 0x0d
-#endif
-#ifndef SUBLANG_ARABIC_UAE
-#define SUBLANG_ARABIC_UAE 0x0e
-#endif
-#ifndef SUBLANG_ARABIC_BAHRAIN
-#define SUBLANG_ARABIC_BAHRAIN 0x0f
-#endif
-#ifndef SUBLANG_ARABIC_QATAR
-#define SUBLANG_ARABIC_QATAR 0x10
-#endif
-#ifndef SUBLANG_AZERI_LATIN
-#define SUBLANG_AZERI_LATIN 0x01
-#endif
-#ifndef SUBLANG_AZERI_CYRILLIC
-#define SUBLANG_AZERI_CYRILLIC 0x02
-#endif
+
 #ifndef SUBLANG_CHINESE_MACAU
 #define SUBLANG_CHINESE_MACAU 0x05
 #endif
-#ifndef SUBLANG_ENGLISH_SOUTH_AFRICA
-#define SUBLANG_ENGLISH_SOUTH_AFRICA 0x07
-#endif
-#ifndef SUBLANG_ENGLISH_JAMAICA
-#define SUBLANG_ENGLISH_JAMAICA 0x08
-#endif
-#ifndef SUBLANG_ENGLISH_CARIBBEAN
-#define SUBLANG_ENGLISH_CARIBBEAN 0x09
-#endif
-#ifndef SUBLANG_ENGLISH_BELIZE
-#define SUBLANG_ENGLISH_BELIZE 0x0a
-#endif
-#ifndef SUBLANG_ENGLISH_TRINIDAD
-#define SUBLANG_ENGLISH_TRINIDAD 0x0b
-#endif
 #ifndef SUBLANG_ENGLISH_ZIMBABWE
 #define SUBLANG_ENGLISH_ZIMBABWE 0x0c
 #endif
 #ifndef SUBLANG_ENGLISH_PHILIPPINES
 #define SUBLANG_ENGLISH_PHILIPPINES 0x0d
 #endif
-#ifndef SUBLANG_FRENCH_LUXEMBOURG
-#define SUBLANG_FRENCH_LUXEMBOURG 0x05
-#endif
 #ifndef SUBLANG_FRENCH_MONACO
 #define SUBLANG_FRENCH_MONACO 0x06
 #endif
-#ifndef SUBLANG_GERMAN_LUXEMBOURG
-#define SUBLANG_GERMAN_LUXEMBOURG 0x04
-#endif
-#ifndef SUBLANG_GERMAN_LIECHTENSTEIN
-#define SUBLANG_GERMAN_LIECHTENSTEIN 0x05
-#endif
 #ifndef SUBLANG_KASHMIRI_INDIA
 #define SUBLANG_KASHMIRI_INDIA 0x02
 #endif
-#ifndef SUBLANG_MALAY_MALAYSIA
-#define SUBLANG_MALAY_MALAYSIA 0x01
-#endif
 #ifndef SUBLANG_MALAY_BRUNEI_DARUSSALAM
 #define SUBLANG_MALAY_BRUNEI_DARUSSALAM 0x02
 #endif
 #ifndef SUBLANG_NEPALI_INDIA
 #define SUBLANG_NEPALI_INDIA 0x02
 #endif
-#ifndef SUBLANG_SERBIAN_LATIN
-#define SUBLANG_SERBIAN_LATIN 0x02
-#endif
-#ifndef SUBLANG_SERBIAN_CYRILLIC
-#define SUBLANG_SERBIAN_CYRILLIC 0x03
-#endif
-#ifndef SUBLANG_SPANISH_GUATEMALA
-#define SUBLANG_SPANISH_GUATEMALA 0x04
-#endif
-#ifndef SUBLANG_SPANISH_COSTA_RICA
-#define SUBLANG_SPANISH_COSTA_RICA 0x05
-#endif
-#ifndef SUBLANG_SPANISH_PANAMA
-#define SUBLANG_SPANISH_PANAMA 0x06
-#endif
-#ifndef SUBLANG_SPANISH_DOMINICAN_REPUBLIC
-#define SUBLANG_SPANISH_DOMINICAN_REPUBLIC 0x07
-#endif
-#ifndef SUBLANG_SPANISH_VENEZUELA
-#define SUBLANG_SPANISH_VENEZUELA 0x08
-#endif
-#ifndef SUBLANG_SPANISH_COLOMBIA
-#define SUBLANG_SPANISH_COLOMBIA 0x09
-#endif
-#ifndef SUBLANG_SPANISH_PERU
-#define SUBLANG_SPANISH_PERU 0x0a
-#endif
-#ifndef SUBLANG_SPANISH_ARGENTINA
-#define SUBLANG_SPANISH_ARGENTINA 0x0b
-#endif
-#ifndef SUBLANG_SPANISH_ECUADOR
-#define SUBLANG_SPANISH_ECUADOR 0x0c
-#endif
-#ifndef SUBLANG_SPANISH_CHILE
-#define SUBLANG_SPANISH_CHILE 0x0d
-#endif
-#ifndef SUBLANG_SPANISH_URUGUAY
-#define SUBLANG_SPANISH_URUGUAY 0x0e
-#endif
-#ifndef SUBLANG_SPANISH_PARAGUAY
-#define SUBLANG_SPANISH_PARAGUAY 0x0f
-#endif
-#ifndef SUBLANG_SPANISH_BOLIVIA
-#define SUBLANG_SPANISH_BOLIVIA 0x10
-#endif
-#ifndef SUBLANG_SPANISH_EL_SALVADOR
-#define SUBLANG_SPANISH_EL_SALVADOR 0x11
-#endif
-#ifndef SUBLANG_SPANISH_HONDURAS
-#define SUBLANG_SPANISH_HONDURAS 0x12
-#endif
-#ifndef SUBLANG_SPANISH_NICARAGUA
-#define SUBLANG_SPANISH_NICARAGUA 0x13
-#endif
-#ifndef SUBLANG_SPANISH_PUERTO_RICO
-#define SUBLANG_SPANISH_PUERTO_RICO 0x14
-#endif
-#ifndef SUBLANG_SWEDISH_FINLAND
-#define SUBLANG_SWEDISH_FINLAND 0x02
-#endif
 #ifndef SUBLANG_URDU_PAKISTAN
 #define SUBLANG_URDU_PAKISTAN 0x01
 #endif
 #ifndef SUBLANG_URDU_INDIA
 #define SUBLANG_URDU_INDIA 0x02
 #endif
-#ifndef SUBLANG_UZBEK_LATIN
-#define SUBLANG_UZBEK_LATIN 0x01
-#endif
 #ifndef SUBLANG_UZBEK_CYRILLIC
 #define SUBLANG_UZBEK_CYRILLIC 0x02
 #endif
@@ -541,35 +214,44 @@ g_win32_closedir (DIR *dir)
  *
  * The setlocale in the Microsoft C library uses locale names of the
  * form "English_United States.1252" etc. We want the Unixish standard
- * form "en", "zh_TW" etc. This function gets the current thread
- * locale from Windows and returns it as a string of the above form
- * for use in forming file names etc. The returned string should be
- * deallocated with g_free().
+ * form "en_US", "zh_TW" etc. This function gets the current thread
+ * locale from Windows - without any encoding info - and returns it as
+ * a string of the above form for use in forming file names etc. The
+ * returned string should be deallocated with g_free().
  *
- * Returns: allocated locale name
- */
+ * Returns: newly-allocated locale name.
+ **/
 
 gchar *
 g_win32_getlocale (void)
 {
   LCID lcid;
+  LANGID langid;
   gchar *ev;
   gint primary, sub;
   gchar *l = NULL, *sl = NULL;
   gchar bfr[20];
 
-  if ((ev = getenv ("LC_ALL")) != NULL
-      || (ev = getenv ("LC_CTYPE")) != NULL
-      || (ev = getenv ("LANG")) != NULL)
+  /* Let the user override the system settings through environment
+     variables, as on POSIX systems.  */
+  if (((ev = getenv ("LC_ALL")) != NULL && ev[0] != '\0')
+      || ((ev = getenv ("LC_MESSAGES")) != NULL && ev[0] != '\0')
+      || ((ev = getenv ("LANG")) != NULL && ev[0] != '\0'))
     return g_strdup (ev);
 
+  /* Use native Win32 API locale ID.  */
   lcid = GetThreadLocale ();
-  primary = PRIMARYLANGID (LANGIDFROMLCID (lcid));
-  sub = SUBLANGID (LANGIDFROMLCID (lcid));
+
+  /* Strip off the sorting rules, keep only the language part.  */
+  langid = LANGIDFROMLCID (lcid);
+
+  /* Split into language and territory part.  */
+  primary = PRIMARYLANGID (langid);
+  sub = SUBLANGID (langid);
   switch (primary)
     {
-    case LANG_AFRIKAANS: l = "af"; break;
-    case LANG_ALBANIAN: l = "sq"; break;
+    case LANG_AFRIKAANS: l = "af"; sl = "ZA"; break;
+    case LANG_ALBANIAN: l = "sq"; sl = "AL"; break;
     case LANG_ARABIC:
       l = "ar";
       switch (sub)
@@ -592,20 +274,30 @@ g_win32_getlocale (void)
 	case SUBLANG_ARABIC_QATAR: sl = "QA"; break;
 	}
       break;
-    case LANG_ARMENIAN: l = "hy"; break;
-    case LANG_ASSAMESE: l = "as"; break;
-    case LANG_AZERI: l = "az"; break;
-    case LANG_BASQUE: l = "eu"; break;
-    case LANG_BELARUSIAN: l = "be"; break;
-    case LANG_BENGALI: l = "bn"; break;
-    case LANG_BULGARIAN: l = "bg"; break;
-    case LANG_CATALAN: l = "ca"; break;
+    case LANG_ARMENIAN: l = "hy"; sl = "AM"; break;
+    case LANG_ASSAMESE: l = "as"; sl = "IN"; break;
+    case LANG_AZERI:
+      l = "az";
+      switch (sub)
+	{
+	/* FIXME: Adjust this when Azerbaijani locales appear on Unix.  */
+	case SUBLANG_AZERI_LATIN: sl = "@latin"; break;
+	case SUBLANG_AZERI_CYRILLIC: sl = "@cyrillic"; break;
+	}
+      break;
+    case LANG_BASQUE:
+      l = "eu"; /* sl could be "ES" or "FR".  */
+      break;
+    case LANG_BELARUSIAN: l = "be"; sl = "BY"; break;
+    case LANG_BENGALI: l = "bn"; sl = "IN"; break;
+    case LANG_BULGARIAN: l = "bg"; sl = "BG"; break;
+    case LANG_CATALAN: l = "ca"; sl = "ES"; break;
     case LANG_CHINESE:
       l = "zh";
       switch (sub)
 	{
 	case SUBLANG_CHINESE_TRADITIONAL: sl = "TW"; break;
-	case SUBLANG_CHINESE_SIMPLIFIED: sl = "CH"; break;
+	case SUBLANG_CHINESE_SIMPLIFIED: sl = "CN"; break;
 	case SUBLANG_CHINESE_HONGKONG: sl = "HK"; break;
 	case SUBLANG_CHINESE_SINGAPORE: sl = "SG"; break;
 	case SUBLANG_CHINESE_MACAU: sl = "MO"; break;
@@ -621,17 +313,19 @@ g_win32_getlocale (void)
 				 */
       switch (sub)
 	{
-	case SUBLANG_SERBIAN_LATIN: l = "sp"; break;
-	case SUBLANG_SERBIAN_CYRILLIC: l = "sr"; break;
-	default: l = "hr";	/* ??? */
+	/* FIXME: How to distinguish Croatian and Latin Serbian locales?  */
+	case SUBLANG_SERBIAN_LATIN: l = "sr"; break;
+	case SUBLANG_SERBIAN_CYRILLIC: l = "sr"; sl = "@cyrillic"; break;
+	default: l = "hr"; sl = "HR";
 	}
       break;
-    case LANG_CZECH: l = "cs"; break;
-    case LANG_DANISH: l = "da"; break;
+    case LANG_CZECH: l = "cs"; sl = "CZ"; break;
+    case LANG_DANISH: l = "da"; sl = "DK"; break;
     case LANG_DUTCH:
       l = "nl";
       switch (sub)
 	{
+	case SUBLANG_DUTCH: sl = "NL"; break;
 	case SUBLANG_DUTCH_BELGIAN: sl = "BE"; break;
 	}
       break;
@@ -639,10 +333,7 @@ g_win32_getlocale (void)
       l = "en";
       switch (sub)
 	{
-	/* SUBLANG_ENGLISH_US == SUBLANG_DEFAULT. Heh. I thought
-	 * English was the language spoken in England.
-	 * Oh well.
-	 */
+	case SUBLANG_ENGLISH_US: sl = "US"; break;
 	case SUBLANG_ENGLISH_UK: sl = "GB"; break;
 	case SUBLANG_ENGLISH_AUS: sl = "AU"; break;
 	case SUBLANG_ENGLISH_CAN: sl = "CA"; break;
@@ -650,21 +341,22 @@ g_win32_getlocale (void)
 	case SUBLANG_ENGLISH_EIRE: sl = "IE"; break;
 	case SUBLANG_ENGLISH_SOUTH_AFRICA: sl = "ZA"; break;
 	case SUBLANG_ENGLISH_JAMAICA: sl = "JM"; break;
-	case SUBLANG_ENGLISH_CARIBBEAN: sl = "@caribbean"; break; /* ??? */
+	case SUBLANG_ENGLISH_CARIBBEAN: sl = "GD"; break; /* Grenada? */
 	case SUBLANG_ENGLISH_BELIZE: sl = "BZ"; break;
 	case SUBLANG_ENGLISH_TRINIDAD: sl = "TT"; break;
 	case SUBLANG_ENGLISH_ZIMBABWE: sl = "ZW"; break;
 	case SUBLANG_ENGLISH_PHILIPPINES: sl = "PH"; break;
 	}
       break;
-    case LANG_ESTONIAN: l = "et"; break;
-    case LANG_FAEROESE: l = "fo"; break;
-    case LANG_FARSI: l = "fa"; break;
-    case LANG_FINNISH: l = "fi"; break;
+    case LANG_ESTONIAN: l = "et"; sl = "EE"; break;
+    case LANG_FAEROESE: l = "fo"; sl = "FO"; break;
+    case LANG_FARSI: l = "fa"; sl = "IR"; break;
+    case LANG_FINNISH: l = "fi"; sl = "FI"; break;
     case LANG_FRENCH:
       l = "fr";
       switch (sub)
 	{
+	case SUBLANG_FRENCH: sl = "FR"; break;
 	case SUBLANG_FRENCH_BELGIAN: sl = "BE"; break;
 	case SUBLANG_FRENCH_CANADIAN: sl = "CA"; break;
 	case SUBLANG_FRENCH_SWISS: sl = "CH"; break;
@@ -672,60 +364,71 @@ g_win32_getlocale (void)
 	case SUBLANG_FRENCH_MONACO: sl = "MC"; break;
 	}
       break;
-    case LANG_GEORGIAN: l = "ka"; break;
+    case LANG_GEORGIAN: l = "ka"; sl = "GE"; break;
     case LANG_GERMAN:
       l = "de";
       switch (sub)
 	{
+	case SUBLANG_GERMAN: sl = "DE"; break;
 	case SUBLANG_GERMAN_SWISS: sl = "CH"; break;
 	case SUBLANG_GERMAN_AUSTRIAN: sl = "AT"; break;
 	case SUBLANG_GERMAN_LUXEMBOURG: sl = "LU"; break;
 	case SUBLANG_GERMAN_LIECHTENSTEIN: sl = "LI"; break;
 	}
       break;
-    case LANG_GREEK: l = "el"; break;
-    case LANG_GUJARATI: l = "gu"; break;
-    case LANG_HEBREW: l = "he"; break;
-    case LANG_HINDI: l = "hi"; break;
-    case LANG_HUNGARIAN: l = "hu"; break;
-    case LANG_ICELANDIC: l = "is"; break;
-    case LANG_INDONESIAN: l = "id"; break;
+    case LANG_GREEK: l = "el"; sl = "GR"; break;
+    case LANG_GUJARATI: l = "gu"; sl = "IN"; break;
+    case LANG_HEBREW: l = "he"; sl = "IL"; break;
+    case LANG_HINDI: l = "hi"; sl = "IN"; break;
+    case LANG_HUNGARIAN: l = "hu"; sl = "HU"; break;
+    case LANG_ICELANDIC: l = "is"; sl = "IS"; break;
+    case LANG_INDONESIAN: l = "id"; sl = "ID"; break;
     case LANG_ITALIAN:
       l = "it";
       switch (sub)
 	{
+	case SUBLANG_ITALIAN: sl = "IT"; break;
 	case SUBLANG_ITALIAN_SWISS: sl = "CH"; break;
 	}
       break;
-    case LANG_JAPANESE: l = "ja"; break;
-    case LANG_KANNADA: l = "kn"; break;
+    case LANG_JAPANESE: l = "ja"; sl = "JP"; break;
+    case LANG_KANNADA: l = "kn"; sl = "IN"; break;
     case LANG_KASHMIRI:
       l = "ks";
       switch (sub)
 	{
+	case SUBLANG_DEFAULT: sl = "PK"; break;
 	case SUBLANG_KASHMIRI_INDIA: sl = "IN"; break;
 	}
       break;
-    case LANG_KAZAK: l = "kk"; break;
-    case LANG_KONKANI: l = "kok"; break; /* ??? */
-    case LANG_KOREAN: l = "ko"; break;
-    case LANG_LATVIAN: l = "lv"; break;
-    case LANG_LITHUANIAN: l = "lt"; break;
-    case LANG_MACEDONIAN: l = "mk"; break;
+    case LANG_KAZAK: l = "kk"; sl = "KZ"; break;
+    case LANG_KONKANI:
+      /* FIXME: Adjust this when such locales appear on Unix.  */
+      l = "kok"; sl = "IN";
+      break;
+    case LANG_KOREAN: l = "ko"; sl = "KR"; break;
+    case LANG_LATVIAN: l = "lv"; sl = "LV"; break;
+    case LANG_LITHUANIAN: l = "lt"; sl = "LT"; break;
+    case LANG_MACEDONIAN: l = "mk"; sl = "MK"; break;
     case LANG_MALAY:
       l = "ms";
       switch (sub)
 	{
+	case SUBLANG_MALAY_MALAYSIA: sl = "MY"; break;
 	case SUBLANG_MALAY_BRUNEI_DARUSSALAM: sl = "BN"; break;
 	}
       break;
-    case LANG_MALAYALAM: l = "ml"; break;
-    case LANG_MANIPURI: l = "mni"; break;
-    case LANG_MARATHI: l = "mr"; break;
+    case LANG_MALAYALAM: l = "ml"; sl = "IN"; break;
+    case LANG_MANIPURI:
+      /* FIXME: Adjust this when such locales appear on Unix.  */
+      l = "mni"; sl = "IN";
+      break;
+    case LANG_MARATHI: l = "mr"; sl = "IN"; break;
     case LANG_NEPALI:
       l = "ne";
       switch (sub)
 	{
+	case SUBLANG_DEFAULT: sl = "NP"; break;
 	case SUBLANG_NEPALI_INDIA: sl = "IN"; break;
 	}
       break;
@@ -733,33 +436,43 @@ g_win32_getlocale (void)
       l = "no";
       switch (sub)
 	{
-	/* SUBLANG_NORWEGIAN_BOKMAL == SUBLANG_DEFAULT */
-	case SUBLANG_NORWEGIAN_NYNORSK: l = "nn"; break;
+	case SUBLANG_NORWEGIAN_BOKMAL: sl = "NO"; break;
+	case SUBLANG_NORWEGIAN_NYNORSK: l = "nn"; sl = "NO"; break;
 	}
       break;
-    case LANG_ORIYA: l = "or"; break;
-    case LANG_POLISH: l = "pl"; break;
+    case LANG_ORIYA: l = "or"; sl = "IN"; break;
+    case LANG_POLISH: l = "pl"; sl = "PL"; break;
     case LANG_PORTUGUESE:
       l = "pt";
       switch (sub)
 	{
-	/* Hmm. SUBLANG_PORTUGUESE_BRAZILIAN == SUBLANG_DEFAULT */
+	case SUBLANG_PORTUGUESE: sl = "PT"; break;
 	case SUBLANG_PORTUGUESE_BRAZILIAN: sl = "BR"; break;
 	}
       break;
-    case LANG_PUNJABI: l = "pa"; break;
-    case LANG_ROMANIAN: l = "ro"; break;
-    case LANG_RUSSIAN: l = "ru"; break;
-    case LANG_SANSKRIT: l = "sa"; break;
+    case LANG_PUNJABI: l = "pa"; sl = "IN"; break;
+    case LANG_ROMANIAN: l = "ro"; sl = "RO"; break;
+    case LANG_RUSSIAN:
+      l = "ru"; /* sl could be "RU" or "UA".  */
+      break;
+    case LANG_SANSKRIT: l = "sa"; sl = "IN"; break;
     case LANG_SINDHI: l = "sd"; break;
-    case LANG_SLOVAK: l = "sk"; break;
-    case LANG_SLOVENIAN: l = "sl"; break;
+    case LANG_SLOVAK: l = "sk"; sl = "SK"; break;
+    case LANG_SLOVENIAN: l = "sl"; sl = "SI"; break;
+#ifdef LANG_SORBIAN
+    case LANG_SORBIAN:
+      /* FIXME: Adjust this when such locales appear on Unix.  */
+      l = "wen"; sl = "DE";
+      break;
+#endif
     case LANG_SPANISH:
       l = "es";
       switch (sub)
 	{
+	case SUBLANG_SPANISH: sl = "ES"; break;
 	case SUBLANG_SPANISH_MEXICAN: sl = "MX"; break;
-	case SUBLANG_SPANISH_MODERN: sl = "@modern"; break;	/* ??? */
+	case SUBLANG_SPANISH_MODERN:
+	  sl = "ES@modern"; break;	/* not seen on Unix */
 	case SUBLANG_SPANISH_GUATEMALA: sl = "GT"; break;
 	case SUBLANG_SPANISH_COSTA_RICA: sl = "CR"; break;
 	case SUBLANG_SPANISH_PANAMA: sl = "PA"; break;
@@ -784,15 +497,18 @@ g_win32_getlocale (void)
       l = "sv";
       switch (sub)
 	{
+	case SUBLANG_DEFAULT: sl = "SE"; break;
 	case SUBLANG_SWEDISH_FINLAND: sl = "FI"; break;
 	}
       break;
-    case LANG_TAMIL: l = "ta"; break;
+    case LANG_TAMIL:
+      l = "ta"; /* sl could be "IN" or "LK".  */
+      break;
     case LANG_TATAR: l = "tt"; break;
-    case LANG_TELUGU: l = "te"; break;
-    case LANG_THAI: l = "th"; break;
-    case LANG_TURKISH: l = "tr"; break;
-    case LANG_UKRAINIAN: l = "uk"; break;
+    case LANG_TELUGU: l = "te"; sl = "IN"; break;
+    case LANG_THAI: l = "th"; sl = "TH"; break;
+    case LANG_TURKISH: l = "tr"; sl = "TR"; break;
+    case LANG_UKRAINIAN: l = "uk"; sl = "UA"; break;
     case LANG_URDU:
       l = "ur";
       switch (sub)
@@ -805,10 +521,12 @@ g_win32_getlocale (void)
       l = "uz";
       switch (sub)
 	{
-	case SUBLANG_UZBEK_CYRILLIC: sl = "@cyrillic"; break;
+	/* FIXME: Adjust this when Uzbek locales appear on Unix.  */
+	case SUBLANG_UZBEK_LATIN: sl = "UZ@latin"; break;
+	case SUBLANG_UZBEK_CYRILLIC: sl = "UZ@cyrillic"; break;
 	}
       break;
-    case LANG_VIETNAMESE: l = "vi"; break;
+    case LANG_VIETNAMESE: l = "vi"; sl = "VN"; break;
     default: l = "xx"; break;
     }
   strcpy (bfr, l);
@@ -824,16 +542,16 @@ g_win32_getlocale (void)
 
 /**
  * g_win32_error_message:
- * @error: error code
+ * @error: error code.
  *
  * Translate a Win32 error code (as returned by GetLastError()) into
  * the corresponding message. The message is either language neutral,
  * or in the thread's language, or the user's language, the system's
- * langauge, or US English (see docs for FormatMessage). The returned
- * string should be deallocated with g_free().
+ * language, or US English (see docs for <function>FormatMessage()</function>). *
+ * The returned string should be deallocated with g_free().
  *
- * Returns: allocated error message
- */
+ * Returns: newly-allocated error message
+ **/
 gchar *
 g_win32_error_message (gint error)
 {
@@ -863,18 +581,24 @@ static gchar *
 get_package_directory_from_module (gchar *module_name)
 {
   static GHashTable *module_dirs = NULL;
+  G_LOCK_DEFINE_STATIC (module_dirs);
   HMODULE hmodule = NULL;
   gchar *fn;
   gchar *p;
   gchar *result;
 
+  G_LOCK (module_dirs);
+
   if (module_dirs == NULL)
     module_dirs = g_hash_table_new (g_str_hash, g_str_equal);
   
   result = g_hash_table_lookup (module_dirs, module_name ? module_name : "");
       
   if (result)
-    return g_strdup (result);
+    {
+      G_UNLOCK (module_dirs);
+      return g_strdup (result);
+    }
 
   if (module_name)
     {
@@ -885,7 +609,19 @@ get_package_directory_from_module (gchar *module_name)
 
   fn = g_malloc (MAX_PATH);
   if (!GetModuleFileName (hmodule, fn, MAX_PATH))
-    return NULL;
+    {
+      G_UNLOCK (module_dirs);
+      g_free (fn);
+      return NULL;
+    }
+
+  if ((p = strrchr (fn, G_DIR_SEPARATOR)) != NULL)
+    *p = '\0';
+
+  p = strrchr (fn, G_DIR_SEPARATOR);
+  if (p && (g_ascii_strcasecmp (p + 1, "bin") == 0 ||
+	    g_ascii_strcasecmp (p + 1, "lib") == 0))
+    *p = '\0';
 
 #ifdef G_WITH_CYGWIN
   /* In Cygwin we need to have POSIX paths */
@@ -898,55 +634,53 @@ get_package_directory_from_module (gchar *module_name)
   }
 #endif
 
-  if ((p = strrchr (fn, G_DIR_SEPARATOR)) != NULL)
-    *p = '\0';
-
-  p = strrchr (fn, G_DIR_SEPARATOR);
-  if (p && (g_ascii_strcasecmp (p + 1, "bin") == 0 ||
-	    g_ascii_strcasecmp (p + 1, "lib") == 0))
-    *p = '\0';
-
   g_hash_table_insert (module_dirs, module_name ? module_name : "", fn);
 
+  G_UNLOCK (module_dirs);
+
   return g_strdup (fn);
 }
 
 /**
  * g_win32_get_package_installation_directory:
- * @package: An identifier for a software package, or NULL
- * @dll_name: The name of a DLL that a package provides, or NULL
+ * @package: An identifier for a software package, or %NULL
+ * @dll_name: The name of a DLL that a package provides, or %NULL.
  *
  * Try to determine the installation directory for a software package.
  * Typically used by GNU software packages.
  *
  * @package should be a short identifier for the package. Typically it
- * is the same identifier as used for GETTEXT_PACKAGE in software
- * configured accoring to GNU standards. The function first looks in
- * the Windows Registry for the value #InstallationDirectory in the
- * key #HKLM\Software\@package, and if that value exists and is a
- * string, returns that.
+ * is the same identifier as used for
+ * <literal>GETTEXT_PACKAGE</literal> in software configured according
+ * to GNU standards. The function first looks in the Windows Registry
+ * for the value <literal>#InstallationDirectory</literal> in the key
+ * <literal>#HKLM\Software\@package</literal>, and if that value
+ * exists and is a string, returns that.
  *
- * If @package is NULL, or the above value isn't found in the
- * Registry, but @dll_name is non-NULL, it should name a DLL loaded
+ * If @package is %NULL, or the above value isn't found in the
+ * Registry, but @dll_name is non-%NULL, it should name a DLL loaded
  * into the current process. Typically that would be the name of the
  * DLL calling this function, looking for its installation
  * directory. The function then asks Windows what directory that DLL
  * was loaded from. If that directory's last component is "bin" or
  * "lib", the parent directory is returned, otherwise the directory
  * itself. If that DLL isn't loaded, the function proceeds as if
- * @dll_name was NULL.
+ * @dll_name was %NULL.
  *
- * If both @package and @dll_name are NULL, the directory from where
+ * If both @package and @dll_name are %NULL, the directory from where
  * the main executable of the process was loaded is uses instead in
  * the same way as above.
  *
- * The return value should be freed with g_free() when not needed any longer.  */
+ * Returns: a string containing the installation directory for @package.
+ * The return value should be freed with g_free() when not needed any longer.
+ **/
 
 gchar *
 g_win32_get_package_installation_directory (gchar *package,
 					    gchar *dll_name)
 {
   static GHashTable *package_dirs = NULL;
+  G_LOCK_DEFINE_STATIC (package_dirs);
   gchar *result = NULL;
   gchar *key;
   HKEY reg_key = NULL;
@@ -955,13 +689,18 @@ g_win32_get_package_installation_directory (gchar *package,
 
   if (package != NULL)
     {
+      G_LOCK (package_dirs);
+      
       if (package_dirs == NULL)
 	package_dirs = g_hash_table_new (g_str_hash, g_str_equal);
       
       result = g_hash_table_lookup (package_dirs, package);
       
       if (result && result[0])
-	return g_strdup (result);
+	{
+	  G_UNLOCK (package_dirs);
+	  return g_strdup (result);
+	}
       
       key = g_strconcat ("Software\\", package, NULL);
       
@@ -987,12 +726,14 @@ g_win32_get_package_installation_directory (gchar *package,
 	RegCloseKey (reg_key);
       
       g_free (key);
-      
-    }
-  if (result)
-    {
-      g_hash_table_insert (package_dirs, package, result);
-      return g_strdup (result);
+
+      if (result)
+	{
+	  g_hash_table_insert (package_dirs, package, result);
+	  G_UNLOCK (package_dirs);
+	  return g_strdup (result);
+	}
+      G_UNLOCK (package_dirs);
     }
 
   if (dll_name != NULL)
@@ -1006,16 +747,19 @@ g_win32_get_package_installation_directory (gchar *package,
 
 /**
  * g_win32_get_package_installation_subdirectory:
- * @package: An identifier for a software package, or NULL
- * @dll_name: The name of a DLL that a package provides, or NULL
+ * @package: An identifier for a software package, or %NULL.
+ * @dll_name: The name of a DLL that a package provides, or %NULL.
  * @subdir: A subdirectory of the package installation directory.
  *
- * Returns a string containg the path of the subdirectory @subdir in
- * the return value from calling
+ * Returns a newly-allocated string containing the path of the
+ * subdirectory @subdir in the return value from calling
  * g_win32_get_package_installation_directory() with the @package and
- * @dll_name parameters. The return value should be freed with
+ * @dll_name parameters. 
+ *
+ * Returns: a string containing the complete path to @subdir inside the 
+ * installation directory of @package. The return value should be freed with
  * g_free() when no longer needed.
- */
+ **/
 
 gchar *
 g_win32_get_package_installation_subdirectory (gchar *package,
@@ -1027,7 +771,7 @@ g_win32_get_package_installation_subdirectory (gchar *package,
 
   prefix = g_win32_get_package_installation_directory (package, dll_name);
 
-  sep = (prefix[strlen (prefix) - 1] == G_DIR_SEPARATOR ?
+  sep = ((subdir != NULL && strlen (subdir) > 0) || prefix[strlen (prefix) - 1] == G_DIR_SEPARATOR ?
 	 "" : G_DIR_SEPARATOR_S);
 
   return g_strconcat (prefix, sep, subdir, NULL);
