@@ -144,6 +144,8 @@ complete_in_arg (InArgument *iarg)
     { "UINT",		"UINT",		"guint",	"g_value_get_uint",	},
     { "LONG",		"LONG",		"glong",	"g_value_get_long",	},
     { "ULONG",		"ULONG",	"gulong",	"g_value_get_ulong",	},
+    { "INT64",		"INT64",	"gint64",       "g_value_get_int64",	},
+    { "UINT64",		"UINT64",	"guint64",	"g_value_get_uint64",	},
     { "ENUM",		"ENUM",		"gint",		"g_value_get_enum",	},
     { "FLAGS",		"FLAGS",	"guint",	"g_value_get_flags",	},
     { "FLOAT",		"FLOAT",	"gfloat",	"g_value_get_float",	},
@@ -187,6 +189,8 @@ complete_out_arg (OutArgument *oarg)
     { "UINT",		"UINT",		"guint",	"g_value_set_uint",	NULL,			NULL          },
     { "LONG",		"LONG",		"glong",	"g_value_set_long",	NULL,			NULL          },
     { "ULONG",		"ULONG",	"gulong",	"g_value_set_ulong",	NULL,			NULL          },
+    { "INT64",		"INT64",	"gint64",	"g_value_set_int64",	NULL,			NULL          },
+    { "UINT64",		"UINT64",	"guint64",	"g_value_set_uint64",	NULL,			NULL          },
     { "ENUM",		"ENUM",		"gint",		"g_value_set_enum",	NULL,			NULL          },
     { "FLAGS",		"FLAGS",	"guint",	"g_value_set_flags",	NULL,			NULL          },
     { "FLOAT",		"FLOAT",	"gfloat",	"g_value_set_float",	NULL,			NULL          },
