@@ -1045,7 +1045,7 @@ g_object_get_property (GObject	   *object,
 	}
       else if (!g_value_type_transformable (G_PARAM_SPEC_VALUE_TYPE (pspec), G_VALUE_TYPE (value)))
 	{
-	  g_warning ("can't retrive property `%s' of type `%s' as value of type `%s'",
+	  g_warning ("can't retrieve property `%s' of type `%s' as value of type `%s'",
 		     pspec->name,
 		     g_type_name (G_PARAM_SPEC_VALUE_TYPE (pspec)),
 		     G_VALUE_TYPE_NAME (value));
