@@ -142,7 +142,7 @@ static inline Emission*		emission_find		(Emission	 *emission_list,
 							 gpointer	  instance);
 static gint			signal_key_cmp		(gconstpointer	  node1,
 							 gconstpointer	  node2);
-static	      gboolean		signal_emit_R		(SignalNode	 *node,
+static	      gboolean		signal_emit_unlocked_R	(SignalNode	 *node,
 							 GQuark		  detail,
 							 gpointer	  instance,
 							 GValue		 *return_value,
@@ -237,6 +237,8 @@ static GTrashStack   *g_handler_ts = NULL;
 static GTrashStack   *g_emission_ts = NULL;
 static gulong         g_handler_sequential_number = 1;
 G_LOCK_DEFINE_STATIC (g_signal_mutex);
+#define	SIGNAL_LOCK()		G_LOCK (g_signal_mutex)
+#define	SIGNAL_UNLOCK()		G_UNLOCK (g_signal_mutex)
 
 
 /* --- signal nodes --- */
@@ -540,9 +542,9 @@ handler_unref_R (guint    signal_id,
           
           hlist->handlers = handler->next;
         }
-      G_UNLOCK (g_signal_mutex);
+      SIGNAL_UNLOCK ();
       g_closure_unref (handler->closure);
-      G_LOCK (g_signal_mutex);
+      SIGNAL_LOCK ();
       g_generic_node_free (&g_handler_ts, handler);
     }
 }
@@ -647,7 +649,7 @@ signal_key_cmp (gconstpointer node1,
 void
 g_signal_init (void) /* sync with gtype.c */
 {
-  G_LOCK (g_signal_mutex);
+  SIGNAL_LOCK ();
   if (!g_n_signal_nodes)
     {
       /* handler_id_node_prepend() requires this */
@@ -661,7 +663,7 @@ g_signal_init (void) /* sync with gtype.c */
       g_signal_nodes = g_renew (SignalNode*, g_signal_nodes, g_n_signal_nodes);
       g_signal_nodes[0] = NULL;
     }
-  G_UNLOCK (g_signal_mutex);
+  SIGNAL_UNLOCK ();
 }
 
 void
@@ -669,7 +671,7 @@ _g_signals_destroy (GType itype)
 {
   guint i;
   
-  G_LOCK (g_signal_mutex);
+  SIGNAL_LOCK ();
   for (i = 1; i < g_n_signal_nodes; i++)
     {
       SignalNode *node = g_signal_nodes[i];
@@ -684,7 +686,7 @@ _g_signals_destroy (GType itype)
 	    signal_destroy_R (node);
         }
     }
-  G_UNLOCK (g_signal_mutex);
+  SIGNAL_UNLOCK ();
 }
 
 void
@@ -697,12 +699,12 @@ g_signal_stop_emission (gpointer instance,
   g_return_if_fail (G_TYPE_CHECK_INSTANCE (instance));
   g_return_if_fail (signal_id > 0);
   
-  G_LOCK (g_signal_mutex);
+  SIGNAL_LOCK ();
   node = LOOKUP_SIGNAL_NODE (signal_id);
   if (node && detail && !(node->flags & G_SIGNAL_DETAILED))
     {
       g_warning ("%s: signal id `%u' does not support detail (%u)", G_STRLOC, signal_id, detail);
-      G_UNLOCK (g_signal_mutex);
+      SIGNAL_UNLOCK ();
       return;
     }
   if (node && g_type_is_a (G_TYPE_FROM_INSTANCE (instance), node->itype))
@@ -724,7 +726,7 @@ g_signal_stop_emission (gpointer instance,
     }
   else
     g_warning ("%s: signal id `%u' is invalid for instance `%p'", G_STRLOC, signal_id, instance);
-  G_UNLOCK (g_signal_mutex);
+  SIGNAL_UNLOCK ();
 }
 
 static void
@@ -736,9 +738,9 @@ signal_finalize_hook (GHookList *hook_list,
   if (destroy)
     {
       hook->destroy = NULL;
-      G_UNLOCK (g_signal_mutex);
+      SIGNAL_UNLOCK ();
       destroy (hook->data);
-      G_LOCK (g_signal_mutex);
+      SIGNAL_LOCK ();
     }
 }
 
@@ -757,18 +759,18 @@ g_signal_add_emission_hook (guint               signal_id,
   g_return_val_if_fail (signal_id > 0, 0);
   g_return_val_if_fail (hook_func != NULL, 0);
 
-  G_LOCK (g_signal_mutex);
+  SIGNAL_LOCK ();
   node = LOOKUP_SIGNAL_NODE (signal_id);
   if (!node || node->destroyed || (node->flags & G_SIGNAL_NO_HOOKS))
     {
       g_warning ("%s: invalid signal id `%u'", G_STRLOC, signal_id);
-      G_UNLOCK (g_signal_mutex);
+      SIGNAL_UNLOCK ();
       return 0;
     }
   if (detail && !(node->flags & G_SIGNAL_DETAILED))
     {
       g_warning ("%s: signal id `%u' does not support detail (%u)", G_STRLOC, signal_id, detail);
-      G_UNLOCK (g_signal_mutex);
+      SIGNAL_UNLOCK ();
       return 0;
     }
   if (!node->emission_hooks)
@@ -786,7 +788,7 @@ g_signal_add_emission_hook (guint               signal_id,
   node->emission_hooks->seq_id = seq_hook_id;
   g_hook_append (node->emission_hooks, hook);
   seq_hook_id = node->emission_hooks->seq_id;
-  G_UNLOCK (g_signal_mutex);
+  SIGNAL_UNLOCK ();
 
   return hook->hook_id;
 }
@@ -800,13 +802,13 @@ g_signal_remove_emission_hook (guint  signal_id,
   g_return_if_fail (signal_id > 0);
   g_return_if_fail (hook_id > 0);
 
-  G_LOCK (g_signal_mutex);
+  SIGNAL_LOCK ();
   node = LOOKUP_SIGNAL_NODE (signal_id);
   if (!node || node->destroyed)
     g_warning ("%s: invalid signal id `%u'", G_STRLOC, signal_id);
   else if (!node->emission_hooks || !g_hook_destroy (node->emission_hooks, hook_id))
     g_warning ("%s: signal \"%s\" had no hook (%lu) to remove", G_STRLOC, node->name, hook_id);
-  G_UNLOCK (g_signal_mutex);
+  SIGNAL_UNLOCK ();
 }
 
 static inline guint
@@ -867,9 +869,9 @@ g_signal_parse_name (const gchar *detailed_signal,
   g_return_val_if_fail (detailed_signal != NULL, FALSE);
   g_return_val_if_fail (G_TYPE_IS_INSTANTIATABLE (itype) || G_TYPE_IS_INTERFACE (itype), FALSE);
   
-  G_LOCK (g_signal_mutex);
+  SIGNAL_LOCK ();
   signal_id = signal_parse_name (detailed_signal, itype, &detail, force_detail_quark);
-  G_UNLOCK (g_signal_mutex);
+  SIGNAL_UNLOCK ();
 
   node = signal_id ? LOOKUP_SIGNAL_NODE (signal_id) : NULL;
   if (!node || node->destroyed ||
@@ -884,6 +886,51 @@ g_signal_parse_name (const gchar *detailed_signal,
   return TRUE;
 }
 
+void
+g_signal_stop_emission_by_name (gpointer     instance,
+				const gchar *detailed_signal)
+{
+  guint signal_id;
+  GQuark detail = 0;
+  GType itype;
+  
+  g_return_if_fail (G_TYPE_CHECK_INSTANCE (instance));
+  g_return_if_fail (detailed_signal != NULL);
+  
+  SIGNAL_LOCK ();
+  itype = G_TYPE_FROM_INSTANCE (instance);
+  signal_id = signal_parse_name (detailed_signal, itype, &detail, TRUE);
+  if (signal_id)
+    {
+      SignalNode *node = LOOKUP_SIGNAL_NODE (signal_id);
+      
+      if (detail && !(node->flags & G_SIGNAL_DETAILED))
+	g_warning ("%s: signal `%s' does not support details", G_STRLOC, detailed_signal);
+      else if (!g_type_is_a (itype, node->itype))
+	g_warning ("%s: signal `%s' is invalid for instance `%p'", G_STRLOC, detailed_signal, instance);
+      else
+	{
+	  Emission *emission_list = node->flags & G_SIGNAL_NO_RECURSE ? g_restart_emissions : g_recursive_emissions;
+	  Emission *emission = emission_find (emission_list, signal_id, detail, instance);
+	  
+	  if (emission)
+	    {
+	      if (*emission->state_p == EMISSION_HOOK)
+		g_warning (G_STRLOC ": emission of signal \"%s\" for instance `%p' cannot be stopped from emission hook",
+			   node->name, instance);
+	      else if (*emission->state_p == EMISSION_RUN)
+		*emission->state_p = EMISSION_STOP;
+	    }
+	  else
+	    g_warning (G_STRLOC ": no emission of signal \"%s\" to stop for instance `%p'",
+		       node->name, instance);
+	}
+    }
+  else
+    g_warning ("%s: signal `%s' is invalid for instance `%p'", G_STRLOC, detailed_signal, instance);
+  SIGNAL_UNLOCK ();
+}
+
 guint
 g_signal_lookup (const gchar *name,
                  GType        itype)
@@ -893,52 +940,26 @@ g_signal_lookup (const gchar *name,
   g_return_val_if_fail (name != NULL, 0);
   g_return_val_if_fail (G_TYPE_IS_INSTANTIATABLE (itype) || G_TYPE_IS_INTERFACE (itype), 0);
   
-  G_LOCK (g_signal_mutex);
+  SIGNAL_LOCK ();
   signal_id = signal_id_lookup (g_quark_try_string (name), itype);
-  G_UNLOCK (g_signal_mutex);
+  SIGNAL_UNLOCK ();
+  if (!signal_id)
+    {
+      /* give elaborate warnings */
+      if (!g_type_name (itype))
+	g_warning (G_STRLOC ": unable to lookup signal \"%s\" for invalid type id `%u'",
+		   name, itype);
+      else if (!G_TYPE_IS_INSTANTIATABLE (itype))
+	g_warning (G_STRLOC ": unable to lookup signal \"%s\" for non instantiatable type `%s'",
+		   name, g_type_name (itype));
+      else if (!g_type_class_peek (itype))
+	g_warning (G_STRLOC ": unable to lookup signal \"%s\" of unloaded type `%s'",
+		   name, g_type_name (itype));
+    }
   
   return signal_id;
 }
 
-G_CONST_RETURN gchar*
-g_signal_name (guint signal_id)
-{
-  SignalNode *node;
-  gchar *name;
-  
-  G_LOCK (g_signal_mutex);
-  node = LOOKUP_SIGNAL_NODE (signal_id);
-  name = node ? node->name : NULL;
-  G_UNLOCK (g_signal_mutex);
-  
-  return name;
-}
-
-void
-g_signal_query (guint         signal_id,
-		GSignalQuery *query)
-{
-  SignalNode *node;
-  
-  g_return_if_fail (query != NULL);
-  
-  G_LOCK (g_signal_mutex);
-  node = LOOKUP_SIGNAL_NODE (signal_id);
-  if (!node || node->destroyed)
-    query->signal_id = 0;
-  else
-    {
-      query->signal_id = node->signal_id;
-      query->signal_name = node->name;
-      query->itype = node->itype;
-      query->signal_flags = node->flags;
-      query->return_type = node->return_type;
-      query->n_params = node->n_params;
-      query->param_types = node->param_types;
-    }
-  G_UNLOCK (g_signal_mutex);
-}
-
 guint*
 g_signal_list_ids (GType  itype,
 		   guint *n_ids)
@@ -951,8 +972,7 @@ g_signal_list_ids (GType  itype,
   g_return_val_if_fail (G_TYPE_IS_INSTANTIATABLE (itype) || G_TYPE_IS_INTERFACE (itype), NULL);
   g_return_val_if_fail (n_ids != NULL, NULL);
   
-  G_LOCK (g_signal_mutex);
-  
+  SIGNAL_LOCK ();
   keys = g_signal_key_bsa.nodes;
   n_nodes  = g_signal_key_bsa.n_nodes;
   result = g_array_new (FALSE, FALSE, sizeof (guint));
@@ -968,14 +988,64 @@ g_signal_list_ids (GType  itype,
 	if (!strchr (name, '_'))
 	  g_array_append_val (result, keys[i].signal_id);
       }
-  
   *n_ids = result->len;
-  
-  G_UNLOCK (g_signal_mutex);
+  SIGNAL_UNLOCK ();
+  if (!n_nodes)
+    {
+      /* give elaborate warnings */
+      if (!g_type_name (itype))
+	g_warning (G_STRLOC ": unable to list signals for invalid type id `%u'",
+		   itype);
+      else if (!G_TYPE_IS_INSTANTIATABLE (itype))
+	g_warning (G_STRLOC ": unable to list signals of non instantiatable type `%s'",
+		   g_type_name (itype));
+      else if (!g_type_class_peek (itype))
+	g_warning (G_STRLOC ": unable to list signals of unloaded type `%s'",
+		   g_type_name (itype));
+    }
   
   return (guint*) g_array_free (result, FALSE);
 }
 
+G_CONST_RETURN gchar*
+g_signal_name (guint signal_id)
+{
+  SignalNode *node;
+  gchar *name;
+  
+  SIGNAL_LOCK ();
+  node = LOOKUP_SIGNAL_NODE (signal_id);
+  name = node ? node->name : NULL;
+  SIGNAL_UNLOCK ();
+  
+  return name;
+}
+
+void
+g_signal_query (guint         signal_id,
+		GSignalQuery *query)
+{
+  SignalNode *node;
+  
+  g_return_if_fail (query != NULL);
+  
+  SIGNAL_LOCK ();
+  node = LOOKUP_SIGNAL_NODE (signal_id);
+  if (!node || node->destroyed)
+    query->signal_id = 0;
+  else
+    {
+      query->signal_id = node->signal_id;
+      query->signal_name = node->name;
+      query->itype = node->itype;
+      query->signal_flags = node->flags;
+      query->return_type = node->return_type;
+      query->n_params = node->n_params;
+      query->param_types = node->param_types;
+    }
+  SIGNAL_UNLOCK ();
+}
+
 guint
 g_signal_new_valist (const gchar       *signal_name,
                      GType              itype,
@@ -1011,16 +1081,16 @@ g_signal_new_valist (const gchar       *signal_name,
 }
 
 guint
-g_signal_newc (const gchar	 *signal_name,
-               GType		  itype,
-               GSignalFlags	  signal_flags,
-               guint              class_offset,
-	       GSignalAccumulator accumulator,
-	       gpointer		  accu_data,
-               GSignalCMarshaller c_marshaller,
-               GType		  return_type,
-               guint		  n_params,
-               ...)
+g_signal_new (const gchar	 *signal_name,
+	      GType		  itype,
+	      GSignalFlags	  signal_flags,
+	      guint               class_offset,
+	      GSignalAccumulator  accumulator,
+	      gpointer		  accu_data,
+	      GSignalCMarshaller  c_marshaller,
+	      GType		  return_type,
+	      guint		  n_params,
+	      ...)
 {
   va_list args;
   guint signal_id;
@@ -1068,7 +1138,7 @@ g_signal_newv (const gchar       *signal_name,
   name = g_strdup (signal_name);
   g_strdelimit (name, G_STR_DELIMITERS ":^", '_');  /* FIXME do character checks like for types */
   
-  G_LOCK (g_signal_mutex);
+  SIGNAL_LOCK ();
   
   signal_id = signal_id_lookup (g_quark_try_string (name), itype);
   node = LOOKUP_SIGNAL_NODE (signal_id);
@@ -1079,7 +1149,7 @@ g_signal_newv (const gchar       *signal_name,
                  g_type_name (node->itype),
                  G_TYPE_IS_INTERFACE (node->itype) ? "interface" : "class ancestry");
       g_free (name);
-      G_UNLOCK (g_signal_mutex);
+      SIGNAL_UNLOCK ();
       return 0;
     }
   if (node && node->itype != itype)
@@ -1089,7 +1159,7 @@ g_signal_newv (const gchar       *signal_name,
                  g_type_name (itype),
                  g_type_name (node->itype));
       g_free (name);
-      G_UNLOCK (g_signal_mutex);
+      SIGNAL_UNLOCK ();
       return 0;
     }
   for (i = 0; i < n_params; i++)
@@ -1098,7 +1168,7 @@ g_signal_newv (const gchar       *signal_name,
 	g_warning (G_STRLOC ": parameter %d of type `%s' for signal \"%s::%s\" is not a value type",
 		   i + 1, g_type_name (param_types[i] & ~G_SIGNAL_TYPE_STATIC_SCOPE), g_type_name (itype), name);
 	g_free (name);
-	G_UNLOCK (g_signal_mutex);
+	SIGNAL_UNLOCK ();
 	return 0;
       }
   if (return_type != G_TYPE_NONE && !G_TYPE_IS_VALUE (return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE))
@@ -1106,7 +1176,7 @@ g_signal_newv (const gchar       *signal_name,
       g_warning (G_STRLOC ": return value of type `%s' for signal \"%s::%s\" is not a value type",
 		 g_type_name (return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE), g_type_name (itype), name);
       g_free (name);
-      G_UNLOCK (g_signal_mutex);
+      SIGNAL_UNLOCK ();
       return 0;
     }
   if (return_type != G_TYPE_NONE &&
@@ -1116,7 +1186,7 @@ g_signal_newv (const gchar       *signal_name,
 		 g_type_name (itype), name,
 		 g_type_name (return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE));
       g_free (name);
-      G_UNLOCK (g_signal_mutex);
+      SIGNAL_UNLOCK ();
       return 0;
     }
   
@@ -1162,7 +1232,7 @@ g_signal_newv (const gchar       *signal_name,
   node->emission_hooks = NULL;
   if (node->c_marshaller && class_closure && G_CLOSURE_NEEDS_MARSHAL (class_closure))
     g_closure_set_marshal (class_closure, node->c_marshaller);
-  G_UNLOCK (g_signal_mutex);
+  SIGNAL_UNLOCK ();
   return signal_id;
 }
 
@@ -1197,7 +1267,7 @@ signal_destroy_R (SignalNode *signal_node)
   
   /* free contents that need to
    */
-  G_UNLOCK (g_signal_mutex);
+  SIGNAL_UNLOCK ();
   g_free (node.param_types);
   g_closure_unref (node.class_closure);
   g_free (node.accumulator);
@@ -1206,7 +1276,7 @@ signal_destroy_R (SignalNode *signal_node)
       g_hook_list_clear (node.emission_hooks);
       g_free (node.emission_hooks);
     }
-  G_LOCK (g_signal_mutex);
+  SIGNAL_LOCK ();
 }
 
 gulong
@@ -1223,7 +1293,7 @@ g_signal_connect_closure_by_id (gpointer  instance,
   g_return_val_if_fail (signal_id > 0, 0);
   g_return_val_if_fail (closure != NULL, 0);
   
-  G_LOCK (g_signal_mutex);
+  SIGNAL_LOCK ();
   node = LOOKUP_SIGNAL_NODE (signal_id);
   if (node)
     {
@@ -1246,7 +1316,7 @@ g_signal_connect_closure_by_id (gpointer  instance,
     }
   else
     g_warning ("%s: signal id `%u' is invalid for instance `%p'", G_STRLOC, signal_id, instance);
-  G_UNLOCK (g_signal_mutex);
+  SIGNAL_UNLOCK ();
   
   return handler_seq_no;
 }
@@ -1266,7 +1336,7 @@ g_signal_connect_closure (gpointer     instance,
   g_return_val_if_fail (detailed_signal != NULL, 0);
   g_return_val_if_fail (closure != NULL, 0);
 
-  G_LOCK (g_signal_mutex);
+  SIGNAL_LOCK ();
   itype = G_TYPE_FROM_INSTANCE (instance);
   signal_id = signal_parse_name (detailed_signal, itype, &detail, TRUE);
   if (signal_id)
@@ -1292,7 +1362,7 @@ g_signal_connect_closure (gpointer     instance,
     }
   else
     g_warning ("%s: signal `%s' is invalid for instance `%p'", G_STRLOC, detailed_signal, instance);
-  G_UNLOCK (g_signal_mutex);
+  SIGNAL_UNLOCK ();
 
   return handler_seq_no;
 }
@@ -1303,19 +1373,22 @@ g_signal_connect_data (gpointer       instance,
 		       GCallback      c_handler,
 		       gpointer       data,
 		       GClosureNotify destroy_data,
-		       gboolean       swapped,
-		       gboolean       after)
+		       GConnectFlags  connect_flags)
 {
   guint signal_id;
   gulong handler_seq_no = 0;
   GQuark detail = 0;
   GType itype;
-
+  gboolean swapped, after;
+  
   g_return_val_if_fail (G_TYPE_CHECK_INSTANCE (instance), 0);
   g_return_val_if_fail (detailed_signal != NULL, 0);
   g_return_val_if_fail (c_handler != NULL, 0);
 
-  G_LOCK (g_signal_mutex);
+  swapped = (connect_flags & G_CONNECT_SWAPPED) != FALSE;
+  after = (connect_flags & G_CONNECT_AFTER) != FALSE;
+
+  SIGNAL_LOCK ();
   itype = G_TYPE_FROM_INSTANCE (instance);
   signal_id = signal_parse_name (detailed_signal, itype, &detail, TRUE);
   if (signal_id)
@@ -1341,7 +1414,7 @@ g_signal_connect_data (gpointer       instance,
     }
   else
     g_warning ("%s: signal `%s' is invalid for instance `%p'", G_STRLOC, detailed_signal, instance);
-  G_UNLOCK (g_signal_mutex);
+  SIGNAL_UNLOCK ();
 
   return handler_seq_no;
 }
@@ -1355,7 +1428,7 @@ g_signal_handler_block (gpointer instance,
   g_return_if_fail (G_TYPE_CHECK_INSTANCE (instance));
   g_return_if_fail (handler_id > 0);
   
-  G_LOCK (g_signal_mutex);
+  SIGNAL_LOCK ();
   handler = handler_lookup (instance, handler_id, NULL);
   if (handler)
     {
@@ -1367,7 +1440,7 @@ g_signal_handler_block (gpointer instance,
     }
   else
     g_warning ("%s: instance `%p' has no handler with id `%lu'", G_STRLOC, instance, handler_id);
-  G_UNLOCK (g_signal_mutex);
+  SIGNAL_UNLOCK ();
 }
 
 void
@@ -1379,7 +1452,7 @@ g_signal_handler_unblock (gpointer instance,
   g_return_if_fail (G_TYPE_CHECK_INSTANCE (instance));
   g_return_if_fail (handler_id > 0);
   
-  G_LOCK (g_signal_mutex);
+  SIGNAL_LOCK ();
   handler = handler_lookup (instance, handler_id, NULL);
   if (handler)
     {
@@ -1390,7 +1463,7 @@ g_signal_handler_unblock (gpointer instance,
     }
   else
     g_warning ("%s: instance `%p' has no handler with id `%lu'", G_STRLOC, instance, handler_id);
-  G_UNLOCK (g_signal_mutex);
+  SIGNAL_UNLOCK ();
 }
 
 void
@@ -1403,7 +1476,7 @@ g_signal_handler_disconnect (gpointer instance,
   g_return_if_fail (G_TYPE_CHECK_INSTANCE (instance));
   g_return_if_fail (handler_id > 0);
   
-  G_LOCK (g_signal_mutex);
+  SIGNAL_LOCK ();
   handler = handler_lookup (instance, handler_id, &signal_id);
   if (handler)
     {
@@ -1413,7 +1486,25 @@ g_signal_handler_disconnect (gpointer instance,
     }
   else
     g_warning ("%s: instance `%p' has no handler with id `%lu'", G_STRLOC, instance, handler_id);
-  G_UNLOCK (g_signal_mutex);
+  SIGNAL_UNLOCK ();
+}
+
+gboolean
+g_signal_handler_is_connected (gpointer instance,
+			       gulong   handler_id)
+{
+  Handler *handler;
+  gboolean connected;
+
+  g_return_val_if_fail (G_TYPE_CHECK_INSTANCE (instance), FALSE);
+  g_return_val_if_fail (handler_id > 0, FALSE);
+
+  SIGNAL_LOCK ();
+  handler = handler_lookup (instance, handler_id, NULL);
+  connected = handler != NULL;
+  SIGNAL_UNLOCK ();
+
+  return connected;
 }
 
 void
@@ -1423,7 +1514,7 @@ g_signal_handlers_destroy (gpointer instance)
   
   g_return_if_fail (G_TYPE_CHECK_INSTANCE (instance));
   
-  G_LOCK (g_signal_mutex);
+  SIGNAL_LOCK ();
   hlbsa = g_hash_table_lookup (g_handler_list_bsa_ht, instance);
   if (hlbsa)
     {
@@ -1455,7 +1546,7 @@ g_signal_handlers_destroy (gpointer instance)
         }
       g_bsearch_array_destroy (hlbsa);
     }
-  G_UNLOCK (g_signal_mutex);
+  SIGNAL_UNLOCK ();
 }
 
 gulong
@@ -1476,14 +1567,14 @@ g_signal_handler_find (gpointer         instance,
     {
       HandlerMatch *mlist;
       
-      G_LOCK (g_signal_mutex);
+      SIGNAL_LOCK ();
       mlist = handlers_find (instance, mask, signal_id, detail, closure, func, data, TRUE);
       if (mlist)
 	{
 	  handler_seq_no = mlist->handler->sequential_number;
 	  handler_match_free1_R (mlist, instance);
 	}
-      G_UNLOCK (g_signal_mutex);
+      SIGNAL_UNLOCK ();
     }
   
   return handler_seq_no;
@@ -1509,9 +1600,9 @@ signal_handlers_foreach_matched_R (gpointer         instance,
       n_handlers++;
       if (mlist->handler->sequential_number)
 	{
-	  G_UNLOCK (g_signal_mutex);
+	  SIGNAL_UNLOCK ();
 	  callback (instance, mlist->handler->sequential_number);
-	  G_LOCK (g_signal_mutex);
+	  SIGNAL_LOCK ();
 	}
       mlist = handler_match_free1_R (mlist, instance);
     }
@@ -1535,11 +1626,11 @@ g_signal_handlers_block_matched (gpointer         instance,
   
   if (mask & (G_SIGNAL_MATCH_CLOSURE | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA))
     {
-      G_LOCK (g_signal_mutex);
+      SIGNAL_LOCK ();
       n_handlers = signal_handlers_foreach_matched_R (instance, mask, signal_id, detail,
 						      closure, func, data,
 						      g_signal_handler_block);
-      G_UNLOCK (g_signal_mutex);
+      SIGNAL_UNLOCK ();
     }
   
   return n_handlers;
@@ -1561,11 +1652,11 @@ g_signal_handlers_unblock_matched (gpointer         instance,
   
   if (mask & (G_SIGNAL_MATCH_CLOSURE | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA))
     {
-      G_LOCK (g_signal_mutex);
+      SIGNAL_LOCK ();
       n_handlers = signal_handlers_foreach_matched_R (instance, mask, signal_id, detail,
 						      closure, func, data,
 						      g_signal_handler_unblock);
-      G_UNLOCK (g_signal_mutex);
+      SIGNAL_UNLOCK ();
     }
   
   return n_handlers;
@@ -1587,11 +1678,11 @@ g_signal_handlers_disconnect_matched (gpointer         instance,
   
   if (mask & (G_SIGNAL_MATCH_CLOSURE | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA))
     {
-      G_LOCK (g_signal_mutex);
+      SIGNAL_LOCK ();
       n_handlers = signal_handlers_foreach_matched_R (instance, mask, signal_id, detail,
 						      closure, func, data,
 						      g_signal_handler_disconnect);
-      G_UNLOCK (g_signal_mutex);
+      SIGNAL_UNLOCK ();
     }
   
   return n_handlers;
@@ -1609,7 +1700,7 @@ g_signal_has_handler_pending (gpointer instance,
   g_return_val_if_fail (G_TYPE_CHECK_INSTANCE (instance), FALSE);
   g_return_val_if_fail (signal_id > 0, FALSE);
   
-  G_LOCK (g_signal_mutex);
+  SIGNAL_LOCK ();
   if (detail)
     {
       SignalNode *node = LOOKUP_SIGNAL_NODE (signal_id);
@@ -1617,7 +1708,7 @@ g_signal_has_handler_pending (gpointer instance,
       if (!(node->flags & G_SIGNAL_DETAILED))
 	{
 	  g_warning ("%s: signal id `%u' does not support detail (%u)", G_STRLOC, signal_id, detail);
-	  G_UNLOCK (g_signal_mutex);
+	  SIGNAL_UNLOCK ();
 	  return FALSE;
 	}
     }
@@ -1631,7 +1722,7 @@ g_signal_has_handler_pending (gpointer instance,
     }
   else
     has_pending = FALSE;
-  G_UNLOCK (g_signal_mutex);
+  SIGNAL_UNLOCK ();
   
   return has_pending;
 }
@@ -1645,7 +1736,9 @@ g_signal_emitv (const GValue *instance_and_params,
   const GValue *param_values;
   gpointer instance;
   SignalNode *node;
+#ifdef G_ENABLE_DEBUG
   guint i;
+#endif
   
   g_return_if_fail (instance_and_params != NULL);
   instance = g_value_peek_pointer (instance_and_params);
@@ -1654,19 +1747,19 @@ g_signal_emitv (const GValue *instance_and_params,
 
   param_values = instance_and_params + 1;
   
-  G_LOCK (g_signal_mutex);
+  SIGNAL_LOCK ();
   node = LOOKUP_SIGNAL_NODE (signal_id);
   if (!node || !g_type_is_a (G_TYPE_FROM_INSTANCE (instance), node->itype))
     {
       g_warning ("%s: signal id `%u' is invalid for instance `%p'", G_STRLOC, signal_id, instance);
-      G_UNLOCK (g_signal_mutex);
+      SIGNAL_UNLOCK ();
       return;
     }
 #ifdef G_ENABLE_DEBUG
   if (detail && !(node->flags & G_SIGNAL_DETAILED))
     {
       g_warning ("%s: signal id `%u' does not support detail (%u)", G_STRLOC, signal_id, detail);
-      G_UNLOCK (g_signal_mutex);
+      SIGNAL_UNLOCK ();
       return;
     }
   for (i = 0; i < node->n_params; i++)
@@ -1678,7 +1771,7 @@ g_signal_emitv (const GValue *instance_and_params,
 		    i,
 		    node->name,
 		    G_VALUE_TYPE_NAME (param_values + i));
-	G_UNLOCK (g_signal_mutex);
+	SIGNAL_UNLOCK ();
 	return;
       }
   if (node->return_type != G_TYPE_NONE)
@@ -1689,7 +1782,7 @@ g_signal_emitv (const GValue *instance_and_params,
 		      G_STRLOC,
 		      g_type_name (node->return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE),
 		      node->name);
-	  G_UNLOCK (g_signal_mutex);
+	  SIGNAL_UNLOCK ();
 	  return;
 	}
       else if (!node->accumulator && !G_TYPE_CHECK_VALUE_TYPE (return_value, node->return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE))
@@ -1699,7 +1792,7 @@ g_signal_emitv (const GValue *instance_and_params,
 		      g_type_name (node->return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE),
 		      node->name,
 		      G_VALUE_TYPE_NAME (return_value));
-	  G_UNLOCK (g_signal_mutex);
+	  SIGNAL_UNLOCK ();
 	  return;
 	}
     }
@@ -1707,8 +1800,8 @@ g_signal_emitv (const GValue *instance_and_params,
     return_value = NULL;
 #endif	/* G_ENABLE_DEBUG */
 
-  signal_emit_R (node, detail, instance, return_value, instance_and_params);
-  G_UNLOCK (g_signal_mutex);
+  SIGNAL_UNLOCK ();
+  signal_emit_unlocked_R (node, detail, instance, return_value, instance_and_params);
 }
 
 void
@@ -1718,30 +1811,33 @@ g_signal_emit_valist (gpointer instance,
 		      va_list  var_args)
 {
   GValue *instance_and_params, stack_values[MAX_STACK_VALUES], *free_me = NULL;
+  GType signal_return_type;
   GValue *param_values;
   SignalNode *node;
-  guint i;
-
+  guint i, n_params;
+  
   g_return_if_fail (G_TYPE_CHECK_INSTANCE (instance));
   g_return_if_fail (signal_id > 0);
 
-  G_LOCK (g_signal_mutex);
+  SIGNAL_LOCK ();
   node = LOOKUP_SIGNAL_NODE (signal_id);
   if (!node || !g_type_is_a (G_TYPE_FROM_INSTANCE (instance), node->itype))
     {
       g_warning ("%s: signal id `%u' is invalid for instance `%p'", G_STRLOC, signal_id, instance);
-      G_UNLOCK (g_signal_mutex);
+      SIGNAL_UNLOCK ();
       return;
     }
 #ifndef G_DISABLE_CHECKS
   if (detail && !(node->flags & G_SIGNAL_DETAILED))
     {
       g_warning ("%s: signal id `%u' does not support detail (%u)", G_STRLOC, signal_id, detail);
-      G_UNLOCK (g_signal_mutex);
+      SIGNAL_UNLOCK ();
       return;
     }
 #endif  /* !G_DISABLE_CHECKS */
 
+  n_params = node->n_params;
+  signal_return_type = node->return_type;
   if (node->n_params < MAX_STACK_VALUES)
     instance_and_params = stack_values;
   else
@@ -1753,12 +1849,15 @@ g_signal_emit_valist (gpointer instance,
   for (i = 0; i < node->n_params; i++)
     {
       gchar *error;
-
+      GType ptype = node->param_types[i] & ~G_SIGNAL_TYPE_STATIC_SCOPE;
+      gboolean static_scope = node->param_types[i] & G_SIGNAL_TYPE_STATIC_SCOPE;
+      
       param_values[i].g_type = 0;
-      g_value_init (param_values + i, node->param_types[i] & ~G_SIGNAL_TYPE_STATIC_SCOPE);
+      SIGNAL_UNLOCK ();
+      g_value_init (param_values + i, ptype);
       G_VALUE_COLLECT (param_values + i,
 		       var_args,
-		       node->param_types[i] & G_SIGNAL_TYPE_STATIC_SCOPE ? G_VALUE_NOCOPY_CONTENTS : 0,
+		       static_scope ? G_VALUE_NOCOPY_CONTENTS : 0,
 		       &error);
       if (error)
 	{
@@ -1771,26 +1870,31 @@ g_signal_emit_valist (gpointer instance,
 	  while (i--)
 	    g_value_unset (param_values + i);
 
-	  G_UNLOCK (g_signal_mutex);
 	  g_free (free_me);
 	  return;
 	}
+      SIGNAL_LOCK ();
     }
+  SIGNAL_UNLOCK ();
   instance_and_params->g_type = 0;
   g_value_init (instance_and_params, G_TYPE_FROM_INSTANCE (instance));
   g_value_set_instance (instance_and_params, instance);
-  if (node->return_type == G_TYPE_NONE)
-    signal_emit_R (node, detail, instance, NULL, instance_and_params);
+  if (signal_return_type == G_TYPE_NONE)
+    signal_emit_unlocked_R (node, detail, instance, NULL, instance_and_params);
   else
     {
       GValue return_value = { 0, };
       gchar *error = NULL;
+      GType rtype = signal_return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE;
+      gboolean static_scope = signal_return_type & G_SIGNAL_TYPE_STATIC_SCOPE;
+      
+      g_value_init (&return_value, rtype);
+
+      signal_emit_unlocked_R (node, detail, instance, &return_value, instance_and_params);
 
-      g_value_init (&return_value, node->return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE);
-      signal_emit_R (node, detail, instance, &return_value, instance_and_params);
       G_VALUE_LCOPY (&return_value,
 		     var_args,
-		     node->return_type & G_SIGNAL_TYPE_STATIC_SCOPE ? G_VALUE_NOCOPY_CONTENTS : 0,
+		     static_scope ? G_VALUE_NOCOPY_CONTENTS : 0,
 		     &error);
       if (!error)
 	g_value_unset (&return_value);
@@ -1798,18 +1902,17 @@ g_signal_emit_valist (gpointer instance,
 	{
 	  g_warning ("%s: %s", G_STRLOC, error);
 	  g_free (error);
-
+	  
 	  /* we purposely leak the value here, it might not be
 	   * in a sane state if an error condition occoured
 	   */
 	}
     }
-  for (i = 0; i < node->n_params; i++)
+  for (i = 0; i < n_params; i++)
     g_value_unset (param_values + i);
   g_value_unset (instance_and_params);
   if (free_me)
     g_free (free_me);
-  G_UNLOCK (g_signal_mutex);
 }
 
 void
@@ -1836,9 +1939,9 @@ g_signal_emit_by_name (gpointer     instance,
   g_return_if_fail (G_TYPE_CHECK_INSTANCE (instance));
   g_return_if_fail (detailed_signal != NULL);
 
-  G_LOCK (g_signal_mutex);
+  SIGNAL_LOCK ();
   signal_id = signal_parse_name (detailed_signal, G_TYPE_FROM_INSTANCE (instance), &detail, TRUE);
-  G_UNLOCK (g_signal_mutex);
+  SIGNAL_UNLOCK ();
 
   if (signal_id)
     {
@@ -1870,11 +1973,11 @@ accumulate (GSignalInvocationHint *ihint,
 }
 
 static gboolean
-signal_emit_R (SignalNode   *node,
-	       GQuark	     detail,
-	       gpointer      instance,
-	       GValue	    *emission_return,
-	       const GValue *instance_and_params)
+signal_emit_unlocked_R (SignalNode   *node,
+			GQuark	      detail,
+			gpointer      instance,
+			GValue	     *emission_return,
+			const GValue *instance_and_params)
 {
   EmissionState emission_state = 0;
   SignalAccumulator *accumulator;
@@ -1883,7 +1986,7 @@ signal_emit_R (SignalNode   *node,
   HandlerList *hlist;
   Handler *handler_list = NULL;
   GValue *return_accu, accu = { 0, };
-  guint signal_id = node->signal_id;
+  guint signal_id;
   gulong max_sequential_handler_number;
   gboolean return_value_altered = FALSE;
   
@@ -1899,6 +2002,8 @@ signal_emit_R (SignalNode   *node,
     }
 #endif	/* G_ENABLE_DEBUG */
   
+  SIGNAL_LOCK ();
+  signal_id = node->signal_id;
   if (node->flags & G_SIGNAL_NO_RECURSE)
     {
       Emission *emission = emission_find (g_restart_emissions, signal_id, detail, instance);
@@ -1906,6 +2011,7 @@ signal_emit_R (SignalNode   *node,
       if (emission)
 	{
 	  *emission->state_p = EMISSION_RESTART;
+	  SIGNAL_UNLOCK ();
 	  return return_value_altered;
 	}
     }
@@ -1914,8 +2020,10 @@ signal_emit_R (SignalNode   *node,
   accumulator = node->accumulator;
   if (accumulator)
     {
+      SIGNAL_UNLOCK ();
       g_value_init (&accu, node->return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE);
       return_accu = &accu;
+      SIGNAL_LOCK ();
     }
   else
     return_accu = emission_return;
@@ -1939,7 +2047,7 @@ signal_emit_R (SignalNode   *node,
     {
       emission_state = EMISSION_RUN;
       
-      G_UNLOCK (g_signal_mutex);
+      SIGNAL_UNLOCK ();
       g_closure_invoke (class_closure,
 			return_accu,
 			node->n_params + 1,
@@ -1948,7 +2056,7 @@ signal_emit_R (SignalNode   *node,
       if (!accumulate (&ihint, emission_return, &accu, accumulator) &&
 	  emission_state == EMISSION_RUN)
 	emission_state = EMISSION_STOP;
-      G_LOCK (g_signal_mutex);
+      SIGNAL_LOCK ();
       return_value_altered = TRUE;
       
       if (emission_state == EMISSION_STOP)
@@ -1974,9 +2082,9 @@ signal_emit_R (SignalNode   *node,
 	      
 	      was_in_call = G_HOOK_IN_CALL (hook);
 	      hook->flags |= G_HOOK_FLAG_IN_CALL;
-              G_UNLOCK (g_signal_mutex);
+              SIGNAL_UNLOCK ();
 	      need_destroy = !hook_func (&ihint, node->n_params + 1, instance_and_params, hook->data);
-	      G_LOCK (g_signal_mutex);
+	      SIGNAL_LOCK ();
 	      if (!was_in_call)
 		hook->flags &= ~G_HOOK_FLAG_IN_CALL;
 	      if (need_destroy)
@@ -2008,7 +2116,7 @@ signal_emit_R (SignalNode   *node,
 	  else if (!handler->block_count && (!handler->detail || handler->detail == detail) &&
 		   handler->sequential_number < max_sequential_handler_number)
 	    {
-	      G_UNLOCK (g_signal_mutex);
+	      SIGNAL_UNLOCK ();
 	      g_closure_invoke (handler->closure,
 				return_accu,
 				node->n_params + 1,
@@ -2017,7 +2125,7 @@ signal_emit_R (SignalNode   *node,
 	      if (!accumulate (&ihint, emission_return, &accu, accumulator) &&
 		  emission_state == EMISSION_RUN)
 		emission_state = EMISSION_STOP;
-	      G_LOCK (g_signal_mutex);
+	      SIGNAL_LOCK ();
 	      return_value_altered = TRUE;
 	      
 	      tmp = emission_state == EMISSION_RUN ? handler->next : NULL;
@@ -2045,7 +2153,7 @@ signal_emit_R (SignalNode   *node,
     {
       emission_state = EMISSION_RUN;
       
-      G_UNLOCK (g_signal_mutex);
+      SIGNAL_UNLOCK ();
       g_closure_invoke (class_closure,
 			return_accu,
 			node->n_params + 1,
@@ -2054,7 +2162,7 @@ signal_emit_R (SignalNode   *node,
       if (!accumulate (&ihint, emission_return, &accu, accumulator) &&
 	  emission_state == EMISSION_RUN)
 	emission_state = EMISSION_STOP;
-      G_LOCK (g_signal_mutex);
+      SIGNAL_LOCK ();
       return_value_altered = TRUE;
       
       if (emission_state == EMISSION_STOP)
@@ -2076,7 +2184,7 @@ signal_emit_R (SignalNode   *node,
 	  if (handler->after && !handler->block_count && (!handler->detail || handler->detail == detail) &&
 	      handler->sequential_number < max_sequential_handler_number)
 	    {
-	      G_UNLOCK (g_signal_mutex);
+	      SIGNAL_UNLOCK ();
 	      g_closure_invoke (handler->closure,
 				return_accu,
 				node->n_params + 1,
@@ -2085,7 +2193,7 @@ signal_emit_R (SignalNode   *node,
 	      if (!accumulate (&ihint, emission_return, &accu, accumulator) &&
 		  emission_state == EMISSION_RUN)
 		emission_state = EMISSION_STOP;
-	      G_LOCK (g_signal_mutex);
+	      SIGNAL_LOCK ();
 	      return_value_altered = TRUE;
 	      
 	      tmp = emission_state == EMISSION_RUN ? handler->next : NULL;
@@ -2116,7 +2224,7 @@ signal_emit_R (SignalNode   *node,
       
       emission_state = EMISSION_STOP;
       
-      G_UNLOCK (g_signal_mutex);
+      SIGNAL_UNLOCK ();
       if (node->return_type != G_TYPE_NONE && !accumulator)
 	{
 	  g_value_init (&accu, node->return_type & ~G_SIGNAL_TYPE_STATIC_SCOPE);
@@ -2129,7 +2237,7 @@ signal_emit_R (SignalNode   *node,
 			&ihint);
       if (need_unset)
 	g_value_unset (&accu);
-      G_LOCK (g_signal_mutex);
+      SIGNAL_LOCK ();
       
       if (emission_state == EMISSION_RESTART)
 	goto EMIT_RESTART;
@@ -2139,6 +2247,7 @@ signal_emit_R (SignalNode   *node,
     handler_unref_R (signal_id, instance, handler_list);
   
   emission_pop ((node->flags & G_SIGNAL_NO_RECURSE) ? &g_restart_emissions : &g_recursive_emissions, &emission_state);
+  SIGNAL_UNLOCK ();
   if (accumulator)
     g_value_unset (&accu);
   
